/*********************************************************************
 *
 *  Application to Demo HTTP2 Server
 *  Support for HTTP2 module in Microchip TCP/IP Stack
 *   -Implements the application
 *   -Reference: RFC 1002
 *
 *********************************************************************

 * FileName:        CustomHTTPApp.c
 * Dependencies:    TCP/IP stack
 * Processor:       PIC18, PIC24F, PIC24H, dsPIC30F, dsPIC33F, PIC32
 * Compiler:        Microchip C32 v1.05 or higher
 *                  Microchip C30 v3.12 or higher
 *                  Microchip C18 v3.30 or higher
 *                  HI-TECH PICC-18 PRO 9.63PL2 or higher
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * Copyright (C) 2002-2009 Microchip Technology Inc.  All rights
 * reserved.
 *
 * Microchip licenses to you the right to use, modify, copy, and
 * distribute:
 * (i)  the Software when embedded on a Microchip microcontroller or
 *      digital signal controller product ("Device") which is
 *      integrated into Licensee's product; or
 * (ii) ONLY the Software driver source files ENC28J60.c, ENC28J60.h,
 *      ENCX24J600.c and ENCX24J600.h ported to a non-Microchip device
 *      used in conjunction with a Microchip ethernet controller for
 *      the sole purpose of interfacing with the ethernet controller.
 *
 * You should refer to the license agreement accompanying this
 * Software for additional information regarding your rights and
 * obligations.
 *
 * THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * MICROCHIP BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE
 * THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER
 * SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.
 *
 *
 * Author               Date    Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Elliott Wood         6/18/07 Original
 ********************************************************************/
#define __CUSTOMHTTPAPP_C

#include "TCPIPConfig.h"

#if defined(STACK_USE_HTTP2_SERVER)

#include <stdlib.h>
#include "TCPIP Stack/TCPIP.h"
#include "usb_config.h"
#include "./USB/usb.h"
#include "./USB/usb_function_cdc.h"
#include "MainDemo.h"       // Needed for SaveAppConfig() prototype

#define AD_WAIT     10
#define mMIN(a, b)  ((a<b)?a:b)
extern WORD GetAnalog( BYTE ch, BYTE wait );
void set_digital_tris( BYTE num, BYTE hilow );
BOOL CalcSum( BYTE* data, WORD len, BOOL bWrite );

const ROM BYTE* DefaultPortName[MAX_IO_BIT] =
{
    "RA0",
    "RA1",
    "RA3",
    "RF2",
    "RF5",
    "RF6",
    "RH4",
    "RH5",

    "RE0",
    "RE1",
    "RE2",
    "RE3",
    "RE4",
    "RE5",
    "RE6",
    "RE7",

    "RJ0",
    "RJ1",
    "RJ2",
    "RJ3",
    "RJ4",
    "RJ5",
    "RJ6",
    "RJ7"
};

/****************************************************************************
  Section:
    Function Prototypes and Memory Globalizers
  ***************************************************************************/
static HTTP_IO_RESULT HTTPPostPICNICConfig(void);   // PIC-NIC
static HTTP_IO_RESULT HTTPPostLCDSend(void);        // PIC-NIC
static HTTP_IO_RESULT HTTPPostPortDir(void);        // PIC-NIC
static HTTP_IO_RESULT HTTPPostPortNameSet(void);    // PIC-NIC
static HTTP_IO_RESULT HTTPPostAuth(void);           // PIC-NIC
static HTTP_IO_RESULT HTTPPostNTP(void);            // PIC-NIC
static HTTP_IO_RESULT HTTPPostADConf(void);         // PIC-NIC

#if defined(HTTP_USE_POST)
    #if defined(STACK_USE_HTTP_MD5_DEMO)
        #if !defined(STACK_USE_MD5)
            #error The HTTP_MD5_DEMO requires STACK_USE_MD5
        #endif
        static HTTP_IO_RESULT HTTPPostMD5(void);
    #endif
    #if defined(STACK_USE_HTTP_APP_RECONFIG)
        extern APP_CONFIG AppConfig;
        static HTTP_IO_RESULT HTTPPostConfig(void);
        #if defined(STACK_USE_SNMP_SERVER)
        static HTTP_IO_RESULT HTTPPostSNMPCommunity(void);
        #endif
    #endif
    #if defined(STACK_USE_HTTP_EMAIL_DEMO) || defined(STACK_USE_SMTP_CLIENT)
        #if !defined(STACK_USE_SMTP_CLIENT)
            #error The HTTP_EMAIL_DEMO requires STACK_USE_SMTP_CLIENT
        #endif
        static HTTP_IO_RESULT HTTPPostEmail(void);
    #endif
    #if defined(STACK_USE_DYNAMICDNS_CLIENT)
        static HTTP_IO_RESULT HTTPPostDDNSConfig(void);
    #endif
#endif

// RAM allocated for DDNS parameters
#if defined(STACK_USE_DYNAMICDNS_CLIENT)
    static BYTE DDNSData[100];
#endif

// Sticky status message variable.
// This is used to indicated whether or not the previous POST operation was
// successful.  The application uses these to store status messages when a
// POST operation redirects.  This lets the application provide status messages
// after a redirect, when connection instance data has already been lost.
static BOOL lastSuccess = FALSE;

// Stick status message variable.  See lastSuccess for details.
static BOOL lastFailure = FALSE;

static BOOL bIsInitPortName = FALSE;

enum
{
    MODIFIED_NONE = 0,
    MODIFIED_SUCCESS,
    MODIFIED_FAILED
};

static BYTE PasswordModified = MODIFIED_NONE;

/****************************************************************************
  Section:

    Authorization Handlers
  ***************************************************************************/

/*****************************************************************************
  Function:
    BYTE HTTPNeedsAuth(BYTE* cFile)

  Internal:
    See documentation in the TCP/IP Stack API or HTTP2.h for details.
  ***************************************************************************/
#if defined(HTTP_USE_AUTHENTICATION)
BYTE HTTPNeedsAuth(BYTE* cFile)
{
    if ( AppConfig.Flags.bIsAuth )
        return 0x00;

    return 0x80;            // No authentication required
}
#endif

/*****************************************************************************
  Function:
    BYTE HTTPCheckAuth(BYTE* cUser, BYTE* cPass)

  Internal:
    See documentation in the TCP/IP Stack API or HTTP2.h for details.
  ***************************************************************************/
#if defined(HTTP_USE_AUTHENTICATION)
BYTE HTTPCheckAuth(BYTE* cUser, BYTE* cPass)
{
    if(strcmppgm2ram((char *)cUser,(ROM char *)MY_DEFAULT_AUTH_USERNAME) == 0
//        && strcmppgm2ram((char *)cPass, (ROM char *)"microchip") == 0)
        && strcmp((char *)cPass, AuthConfig.password ) == 0)
        return 0x80;        // We accept this combination

    // You can add additional user/pass combos here.
    // If you return specific "realm" values above, you can base this
    //   decision on what specific file or folder is being accessed.
    // You could return different values (0x80 to 0xff) to indicate
    //   various users or groups, and base future processing decisions
    //   in HTTPExecuteGet/Post or HTTPPrint callbacks on this value.

    return 0x00;            // Provided user/pass is invalid
}
#endif

/****************************************************************************
  Section:
    GET Form Handlers
  ***************************************************************************/

/*****************************************************************************
  Function:
    HTTP_IO_RESULT HTTPExecuteGet(void)

  Internal:
    See documentation in the TCP/IP Stack API or HTTP2.h for details.
  ***************************************************************************/
HTTP_IO_RESULT HTTPExecuteGet(void)
{
    BYTE *ptr;
    BYTE filename[20];

    DEBUGLOGRS_UART("HTTPExecuteGet\r\n");

    // Load the file name
    // Make sure BYTE filename[] above is large enough for your longest name
    if ( MPFSGetFilename(curHTTP.file, filename, 20) )
    {
        DEBUGLOGS_UART( filename );
        DEBUGLOGRS_UART( "\r\n" );
    }

    if( 0 == memcmppgm2ram( filename, "modify.cgi", 10) ) // noda
    {
        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P0");
        if(ptr)
        {
            PIO_LOW_O0 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P1");
        if(ptr)
        {
            PIO_LOW_O1 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P2");
        if(ptr)
        {
            PIO_LOW_O2 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P3");
        if(ptr)
        {
            PIO_LOW_O3 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P4");
        if(ptr)
        {
            PIO_LOW_O4 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P5");
        if(ptr)
        {
            PIO_LOW_O5 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P6");
        if(ptr)
        {
            PIO_LOW_O6 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P7");
        if(ptr)
        {
            PIO_LOW_O7 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P8");
        if(ptr)
        {
            PIO_HIGH_O0 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P9");
        if(ptr)
        {
            PIO_HIGH_O1 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P10");
        if(ptr)
        {
            PIO_HIGH_O2 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P11");
        if(ptr)
        {
            PIO_HIGH_O3 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P12");
        if(ptr)
        {
            PIO_HIGH_O4 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P13");
        if(ptr)
        {
            PIO_HIGH_O5 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P14");
        if(ptr)
        {
            PIO_HIGH_O6 = (*ptr == 'H');
        }

        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"P15");
        if(ptr)
        {
            PIO_HIGH_O7 = (*ptr == 'H');
        }

        strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/index.html");
        curHTTP.data[11] = 0x00;
        curHTTP.httpStatus = HTTP_REDIRECT;

    }
    else if( 0 == memcmppgm2ram( filename, "config.cgi", 10) ) // noda
    {
        ptr = HTTPGetROMArg(curHTTP.data, (ROM BYTE*)"go_conf");
        if(ptr)
        {
            strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/config.html");
            curHTTP.data[12] = 0x00;
        }
        else
        {
            strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/index.html");
            curHTTP.data[11] = 0x00;
        }

        curHTTP.httpStatus = HTTP_REDIRECT;
    }

    return HTTP_IO_DONE;
}


/****************************************************************************
  Section:
    POST Form Handlers
  ***************************************************************************/
#if defined(HTTP_USE_POST)

/*****************************************************************************
  Function:
    HTTP_IO_RESULT HTTPExecutePost(void)

  Internal:
    See documentation in the TCP/IP Stack API or HTTP2.h for details.
  ***************************************************************************/
HTTP_IO_RESULT HTTPExecutePost(void)
{
    // Resolve which function to use and pass along
    BYTE filename[20];

    // Load the file name
    // Make sure BYTE filename[] above is large enough for your longest name
    MPFSGetFilename(curHTTP.file, filename, sizeof(filename));

#if defined(STACK_USE_HTTP_MD5_DEMO)
    if(!memcmppgm2ram(filename, "upload.htm", 10))
        return HTTPPostMD5();
#endif

#if defined(STACK_USE_HTTP_APP_RECONFIG)
    if(!memcmppgm2ram(filename, "protect/config.htm", 18))
        return HTTPPostConfig();
    #if defined(STACK_USE_SNMP_SERVER)
    else if(!memcmppgm2ram(filename, "snmp/snmpconfig.htm", 19))
        return HTTPPostSNMPCommunity();
    #endif
#endif

#if defined(STACK_USE_SMTP_CLIENT)
    if(!strcmppgm2ram((char*)filename, (ROM char*)"email/index.htm"))
        return HTTPPostEmail();
#endif

#if defined(STACK_USE_DYNAMICDNS_CLIENT)
    if(!strcmppgm2ram((char*)filename, "dyndns/index.htm"))
        return HTTPPostDDNSConfig();
#endif

    // PIC-NIC
    if(!strcmppgm2ram((char*)filename, "config.cgi"))
        return HTTPPostPICNICConfig();

    if(!strcmppgm2ram((char*)filename, "lcd.cgi"))
        return HTTPPostLCDSend();

    if(!strcmppgm2ram((char*)filename, "modify2.cgi"))
        return HTTPPostPortDir();

    if(!strcmppgm2ram((char*)filename, "config2.cgi"))
        return HTTPPostPortNameSet();

    if(!strcmppgm2ram((char*)filename, "auth.cgi"))
        return HTTPPostAuth();

    if(!strcmppgm2ram((char*)filename, "ntp.cgi"))
        return HTTPPostNTP();

    if(!strcmppgm2ram((char*)filename, "adc.cgi"))
        return HTTPPostADConf();

    return HTTP_IO_DONE;
}

/*****************************************************************************
  Function:
    static HTTP_IO_RESULT HTTPPostConfig(void)

  Summary:
    Processes the configuration form on config/index.htm

  Description:
    Accepts configuration parameters from the form, saves them to a
    temporary location in RAM, then eventually saves the data to EEPROM or
    external Flash.

    When complete, this function redirects to config/reboot.htm, which will
    display information on reconnecting to the board.

    This function creates a shadow copy of the AppConfig structure in
    RAM and then overwrites incoming data there as it arrives.  For each
    name/value pair, the name is first read to curHTTP.data[0:5].  Next, the
    value is read to newAppConfig.  Once all data has been read, the new
    AppConfig is saved back to EEPROM and the browser is redirected to
    reboot.htm.  That file includes an AJAX call to reboot.cgi, which
    performs the actual reboot of the machine.

    If an IP address cannot be parsed, too much data is POSTed, or any other
    parsing error occurs, the browser reloads config.htm and displays an error
    message at the top.

  Precondition:
    None

  Parameters:
    None

  Return Values:
    HTTP_IO_DONE - all parameters have been processed
    HTTP_IO_NEED_DATA - data needed by this function has not yet arrived
  ***************************************************************************/
/*
#if defined(STACK_USE_HTTP_APP_RECONFIG)
static HTTP_IO_RESULT HTTPPostConfig(void)
{
    APP_CONFIG newAppConfig;
    BYTE *ptr;
    BYTE i;

    // Check to see if the browser is attempting to submit more data than we
    // can parse at once.  This function needs to receive all updated
    // parameters and validate them all before committing them to memory so that
    // orphaned configuration parameters do not get written (for example, if a
    // static IP address is given, but the subnet mask fails parsing, we
    // should not use the static IP address).  Everything needs to be processed
    // in a single transaction.  If this is impossible, fail and notify the user.
    // As a web devloper, if you add parameters to AppConfig and run into this
    // problem, you could fix this by to splitting your update web page into two
    // seperate web pages (causing two transactional writes).  Alternatively,
    // you could fix it by storing a static shadow copy of AppConfig someplace
    // in memory and using it instead of newAppConfig.  Lastly, you could
    // increase the TCP RX FIFO size for the HTTP server.  This will allow more
    // data to be POSTed by the web browser before hitting this limit.
    if(curHTTP.byteCount > TCPIsGetReady(sktHTTP) + TCPGetRxFIFOFree(sktHTTP))
        goto ConfigFailure;

    // Ensure that all data is waiting to be parsed.  If not, keep waiting for
    // all of it to arrive.
    if(TCPIsGetReady(sktHTTP) < curHTTP.byteCount)
        return HTTP_IO_NEED_DATA;


    // Use current config in non-volatile memory as defaults
    #if defined(EEPROM_CS_TRIS)
        XEEReadArray(0x0001, (BYTE*)&newAppConfig, sizeof(AppConfig));
    #elif defined(SPIFLASH_CS_TRIS)
        SPIFlashReadArray(0x0001, (BYTE*)&newAppConfig, sizeof(AppConfig));
    #endif

    // Start out assuming that DHCP is disabled.  This is necessary since the
    // browser doesn't submit this field if it is unchecked (meaning zero).
    // However, if it is checked, this will be overridden since it will be
    // submitted.
    newAppConfig.Flags.bIsDHCPEnabled = 0;


    // Read all browser POST data
    while(curHTTP.byteCount)
    {
        // Read a form field name
        if(HTTPReadPostName(curHTTP.data, 6) != HTTP_READ_OK)
            goto ConfigFailure;

        // Read a form field value
        if(HTTPReadPostValue(curHTTP.data + 6, sizeof(curHTTP.data)-6-2) != HTTP_READ_OK)
            goto ConfigFailure;

        // Parse the value that was read
        if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"ip"))
        {// Read new static IP Address
            if(!StringToIPAddress(curHTTP.data+6, &newAppConfig.MyIPAddr))
                goto ConfigFailure;

            newAppConfig.DefaultIPAddr.Val = newAppConfig.MyIPAddr.Val;
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"gw"))
        {// Read new gateway address
            if(!StringToIPAddress(curHTTP.data+6, &newAppConfig.MyGateway))
                goto ConfigFailure;

        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"sub"))
        {// Read new static subnet
            if(!StringToIPAddress(curHTTP.data+6, &newAppConfig.MyMask))
                goto ConfigFailure;

            newAppConfig.DefaultMask.Val = newAppConfig.MyMask.Val;
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"mac"))
        {
            // Read new MAC address
            WORD_VAL w;
            BYTE i;

            ptr = curHTTP.data+6;

            for(i = 0; i < 12u; i++)
            {// Read the MAC address

                // Skip non-hex bytes
                while( *ptr != 0x00u && !(*ptr >= '0' && *ptr <= '9') && !(*ptr >= 'A' && *ptr <= 'F') && !(*ptr >= 'a' && *ptr <= 'f') )
                    ptr++;

                // MAC string is over, so zeroize the rest
                if(*ptr == 0x00u)
                {
                    for(; i < 12u; i++)
                        curHTTP.data[i] = '0';
                    break;
                }

                // Save the MAC byte
                curHTTP.data[i] = *ptr++;
            }

            // Read MAC Address, one byte at a time
            for(i = 0; i < 6u; i++)
            {
                w.v[1] = curHTTP.data[i*2];
                w.v[0] = curHTTP.data[i*2+1];
                newAppConfig.MyMACAddr.v[i] = hexatob(w);
            }
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"host"))
        {// Read new hostname
            FormatNetBIOSName(&curHTTP.data[6]);
            memcpy((void*)newAppConfig.NetBIOSName, (void*)curHTTP.data+6, 16);
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"dhcp"))
        {// Read new DHCP Enabled flag
            if(curHTTP.data[6] == '1')
                newAppConfig.Flags.bIsDHCPEnabled = 1;
        }
    }

    // All parsing complete!  Save new settings and force a reboot
    #if defined(EEPROM_CS_TRIS)
        XEEBeginWrite(0x0000);
        XEEWrite(0x60);
        XEEWriteArray((BYTE*)&newAppConfig, sizeof(AppConfig));
    #elif defined(SPIFLASH_CS_TRIS)
        SPIFlashBeginWrite(0x0000);
        SPIFlashWrite(0x60);
        SPIFlashWriteArray((BYTE*)&newAppConfig, sizeof(AppConfig));
    #endif

    // Set the board to reboot and display reconnecting information
    strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/protect/reboot.htm?");
    memcpy((void*)(curHTTP.data+20), (void*)newAppConfig.NetBIOSName, 16);
    curHTTP.data[20+16] = 0x00; // Force null termination
    for(i = 20; i < 20u+16u; i++)
    {
        if(curHTTP.data[i] == ' ')
            curHTTP.data[i] = 0x00;
    }
    curHTTP.httpStatus = HTTP_REDIRECT;

    return HTTP_IO_DONE;


ConfigFailure:
    lastFailure = TRUE;
    strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/protect/config.htm");
    curHTTP.httpStatus = HTTP_REDIRECT;

    return HTTP_IO_DONE;
}

#endif  // #if defined(STACK_USE_HTTP_APP_RECONFIG)
*/
/*****************************************************************************
  Function:
    static HTTP_IO_RESULT HTTPPostMD5(void)

  Summary:
    Processes the file upload form on upload.htm

  Description:
    This function demonstrates the processing of file uploads.  First, the
    function locates the file data, skipping over any headers that arrive.
    Second, it reads the file 64 bytes at a time and hashes that data.  Once
    all data has been received, the function calculates the MD5 sum and
    stores it in curHTTP.data.

    After the headers, the first line from the form will be the MIME
    separator.  Following that is more headers about the file, which we
    discard.  After another CRLFCRLF, the file data begins, and we read
    it 16 bytes at a time and add that to the MD5 calculation.  The reading
    terminates when the separator string is encountered again on its own
    line.  Notice that the actual file data is trashed in this process,
    allowing us to accept files of arbitrary size, not limited by RAM.
    Also notice that the data buffer is used as an arbitrary storage array
    for the result.  The ~uploadedmd5~ callback reads this data later to
    send back to the client.

  Precondition:
    None

  Parameters:
    None

  Return Values:
    HTTP_IO_DONE - all parameters have been processed
    HTTP_IO_WAITING - the function is pausing to continue later
    HTTP_IO_NEED_DATA - data needed by this function has not yet arrived
  ***************************************************************************/
#if defined(STACK_USE_HTTP_MD5_DEMO)
static HTTP_IO_RESULT HTTPPostMD5(void)
{
    WORD lenA, lenB;
    static HASH_SUM md5;            // Assume only one simultaneous MD5

    #define SM_MD5_READ_SEPARATOR   (0u)
    #define SM_MD5_SKIP_TO_DATA     (1u)
    #define SM_MD5_READ_DATA        (2u)
    #define SM_MD5_POST_COMPLETE    (3u)

    // Don't care about curHTTP.data at this point, so use that for buffer
    switch(curHTTP.smPost)
    {
        // Just started, so try to find the separator string
        case SM_MD5_READ_SEPARATOR:
            // Reset the MD5 calculation
            MD5Initialize(&md5);

            // See if a CRLF is in the buffer
            lenA = TCPFindROMArray(sktHTTP, (ROM BYTE*)"\r\n", 2, 0, FALSE);
            if(lenA == 0xffff)
            {//if not, ask for more data
                return HTTP_IO_NEED_DATA;
            }

            // If so, figure out where the last byte of data is
            // Data ends at CRLFseparator--CRLF, so 6+len bytes
            curHTTP.byteCount -= lenA + 6;

            // Read past the CRLF
            curHTTP.byteCount -= TCPGetArray(sktHTTP, NULL, lenA+2);

            // Save the next state (skip to CRLFCRLF)
            curHTTP.smPost = SM_MD5_SKIP_TO_DATA;

            // No break...continue reading the headers if possible

        // Skip the headers
        case SM_MD5_SKIP_TO_DATA:
            // Look for the CRLFCRLF
            lenA = TCPFindROMArray(sktHTTP, (ROM BYTE*)"\r\n\r\n", 4, 0, FALSE);

            if(lenA != 0xffff)
            {// Found it, so remove all data up to and including
                lenA = TCPGetArray(sktHTTP, NULL, lenA+4);
                curHTTP.byteCount -= lenA;
                curHTTP.smPost = SM_MD5_READ_DATA;
            }
            else
            {// Otherwise, remove as much as possible
                lenA = TCPGetArray(sktHTTP, NULL, TCPIsGetReady(sktHTTP) - 4);
                curHTTP.byteCount -= lenA;

                // Return the need more data flag
                return HTTP_IO_NEED_DATA;
            }

            // No break if we found the header terminator

        // Read and hash file data
        case SM_MD5_READ_DATA:
            // Find out how many bytes are available to be read
            lenA = TCPIsGetReady(sktHTTP);
            if(lenA > curHTTP.byteCount)
                lenA = curHTTP.byteCount;

            while(lenA > 0u)
            {// Add up to 64 bytes at a time to the sum
                lenB = TCPGetArray(sktHTTP, curHTTP.data, (lenA < 64u)?lenA:64);
                curHTTP.byteCount -= lenB;
                lenA -= lenB;
                MD5AddData(&md5, curHTTP.data, lenB);
            }

            // If we've read all the data
            if(curHTTP.byteCount == 0u)
            {// Calculate and copy result to curHTTP.data for printout
                curHTTP.smPost = SM_MD5_POST_COMPLETE;
                MD5Calculate(&md5, curHTTP.data);
                return HTTP_IO_DONE;
            }

            // Ask for more data
            return HTTP_IO_NEED_DATA;
    }

    return HTTP_IO_DONE;
}
#endif // #if defined(STACK_USE_HTTP_MD5_DEMO)


/****************************************************************************
  Section:
    Dynamic Variable Callback Functions
  ***************************************************************************/

/*****************************************************************************
  Function:
    void HTTPPrint_varname(void)

  Internal:
    See documentation in the TCP/IP Stack API or HTTP2.h for details.
  ***************************************************************************/

void HTTPPrint_builddate(void)
{
    curHTTP.callbackPos = 0x01;
    if(TCPIsPutReady(sktHTTP) < strlenpgm((ROM char*)__DATE__" "__TIME__))
        return;

    curHTTP.callbackPos = 0x00;
    TCPPutROMString(sktHTTP, (ROM void*)__DATE__" "__TIME__);
}

void HTTPPrint_version(void)
{
    TCPPutROMString(sktHTTP, (ROM void*)VERSION);
}

//PIC-NIC
void HTTPPrint_GetMaxLengthPortName( void )
{
    BYTE str[5];

    sprintf( str, "%d", PORTNAME_LEN );
    TCPPutString( sktHTTP, str );
    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_GetMaxLengthAuth( void )
{
    BYTE str[5];

    sprintf( str, "%d", AUTH_LEN );
    TCPPutString( sktHTTP, str );
    curHTTP.callbackPos = 0x00;
}

BYTE get_digital_tris( BYTE num )
{
    BYTE trisbit;

    if ( 0 <= num && num <= 7 )
    {
        trisbit = PIO_LOW_TRIS & (1 << num);
    }
    else if ( 8 <= num && num <= 15 )
    {
        trisbit = PIO_HIGH_TRIS & (1 << (num-8));
    }
    else
    {
        trisbit = 0xff;
    }

    return trisbit;
}

void set_digital_tris( BYTE num, BYTE hilow )
{
    BYTE trisbit;

    if ( 0 <= num && num <= 7 )
    {
        if ( hilow )
        {
            AppConfig.TRIS[0] |= (1 << num);
        }
        else
        {
            AppConfig.TRIS[0] &= ~(1 << num);
        }
        PIO_LOW_TRIS = AppConfig.TRIS[0];
    }
    else if ( 8 <= num && num <= 15 )
    {
        num -= 8;

        if ( hilow )
        {
            AppConfig.TRIS[1] |= (1 << num);
        }
        else
        {
            AppConfig.TRIS[1] &= ~(1 << num);
        }
        PIO_HIGH_TRIS = AppConfig.TRIS[1];
    }
}

void HTTPPrint_BoardVersion()
{
    BYTE str[6];
    sprintf( str, "%d.%d", PICNIC_VER1, PICNIC_VER2 );

    TCPPutString( sktHTTP, str );
    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_AnalogInput( WORD ch )
{
    WORD wAn;
    BYTE str[5];

    wAn = GetAnalog( (BYTE)ch, AD_WAIT );

    if ( TCPIsPutReady(sktHTTP) < 4u )
    {
        curHTTP.callbackPos = 0x01;
        return;
    }

    sprintf( str, "%4d", wAn );
    TCPPutString( sktHTTP, str );
    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_dhcp_sw()
{
    if ( AppConfig.Flags.bIsDHCPEnabled )
    {
        TCPPutROMString( sktHTTP, (ROM void*)"checked" );
    }
    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_vref_checked( WORD num, WORD hilow )
{
    BOOL checked = FALSE;

    if ( 0 == num )
    {
        if ( !AppConfig.ADCONF.VCFG0 )
        {
            if ( hilow )
            {
                checked = TRUE;
            }
        }
        else
        {
            if ( !hilow )
            {
                checked = TRUE;
            }
        }
    }
    else
    {
        if ( !AppConfig.ADCONF.VCFG1 )
        {
            if ( hilow )
            {
                checked = TRUE;
            }
        }
        else
        {
            if ( !hilow )
            {
                checked = TRUE;
            }
        }
    }

    if ( checked )
    {
        TCPPutROMString( sktHTTP, (ROM void*)"checked" );
    }
    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_dir_checked( WORD num, WORD hilow )
{
    BYTE trisbit;

    if ( 0 <= num && num < MAX_IO_BIT )
    {
        trisbit = get_digital_tris( (BYTE)num );

        if ( trisbit )
        {
            if ( hilow )
            {
                TCPPutROMString( sktHTTP, (ROM void*)"checked" );
            }
        }
        else
        {
            if ( !hilow )
            {
                TCPPutROMString( sktHTTP, (ROM void*)"checked" );
            }
        }
    }
    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_auth_onoff( void )
{
    if(AppConfig.Flags.bIsAuth)
        TCPPutROMString(sktHTTP, (ROM BYTE*)"checked");
    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_PasswordResult( void )
{
    switch ( PasswordModified )
    {
    case MODIFIED_SUCCESS:
        TCPPutROMString(sktHTTP, (ROM BYTE*)"Change Success");
        break;
    case MODIFIED_FAILED:
        TCPPutROMString(sktHTTP, (ROM BYTE*)"Change Failed");
        break;
    case MODIFIED_NONE:
    default:
        break;
    }
    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_Password( WORD num )
{
//    TCPPutString( sktHTTP, AuthConfig.password );
    TCPPutROMString( sktHTTP, (ROM BYTE*)"****" );
    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_NTPserver( void )
{
    TCPPutString( sktHTTP, gNTPServerName.name );
    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_GetMaxLengthNTP( void )
{
    BYTE str[5];

    sprintf( str, "%d", sizeof(gNTPServerName.name) );
    TCPPutString( sktHTTP, str );
    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_PortName( WORD num )
{
    PORT_NAME   PortName;
    WORD        addr;
    WORD        i;
    WORD        t1;

    if ( 0 <= num && num < MAX_IO_BIT )
    {
    #if defined(EEPROM_CS_TRIS)
        addr = PORTNAME_OFFSET + (num * PORTNAME_BLOCK);

        if ( bIsInitPortName )
        {
            for ( i =0; i < MAX_IO_BIT; i++ )
            {
                for ( t1 = 0; t1 < 10; t1++ )
                {
//                    StackTask(); //wait
                    TickGet();
                }
                memcpypgm2ram( (void*)PortName.name, (ROM void*)DefaultPortName[i], strlenpgm(DefaultPortName[i])+1 );
                CalcSum( (BYTE*)&PortName, sizeof(PortName) - sizeof(PortName.sum), TRUE );
                XEEBeginWrite( (WORD)(PORTNAME_OFFSET + (i * PORTNAME_BLOCK)) );
                XEEWriteArray( (BYTE*)&PortName, sizeof(PortName) );
                DelayMs( 10 );
            }
            bIsInitPortName = FALSE;

            memcpypgm2ram( (void*)PortName.name, (ROM void*)DefaultPortName[num], strlenpgm(DefaultPortName[num])+1 );
        }
        else
        {
            XEEReadArray( addr, (BYTE*)&PortName, sizeof(PortName) );

            if ( CalcSum( (BYTE*)&PortName, sizeof(PortName) - sizeof(PortName.sum), FALSE ) )
            {
                PortName.name[PORTNAME_LEN - 1] = '\0'; // string limit
            }
            else
            {
                memcpypgm2ram( (void*)PortName.name, (ROM void*)DefaultPortName[num], strlenpgm(DefaultPortName[num])+1 );
                CalcSum( (BYTE*)&PortName, sizeof(PortName) - sizeof(PortName.sum), TRUE );
                XEEBeginWrite( addr );
                XEEWriteArray( (BYTE*)&PortName, sizeof(PortName) );
            }
        }
    #else
        memcpypgm2ram( (void*)PortName.name, (ROM void*)DefaultPortName[num], strlenpgm(DefaultPortName[num])+1 );
    #endif
        TCPPutString( sktHTTP, PortName.name );
    }

    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_DigitalTRIS( WORD num )
{
    BYTE trisbit;

    trisbit = get_digital_tris( (BYTE)num );

    if ( 0xff != trisbit )
    {
        TCPPutROMString( sktHTTP, trisbit ? (ROM void*)"In" : (ROM void*)"Out" );
    }

    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_AnalogTRIS( WORD num )
{
    BYTE trisbit;

    switch ( num )
    {
    case 0:
        trisbit = TRISA & (1 << 0);
        break;

    case 1:
        trisbit = TRISA & (1 << 1);
        break;

    case 2:
        trisbit = TRISA & (1 << 3);
        break;

    case 3:
        trisbit = TRISF & (1 << 2);
        break;

    case 4:
        trisbit = TRISF & (1 << 5);
        break;

    case 5:
        trisbit = TRISF & (1 << 6);
        break;

    case 6:
        trisbit = TRISH & (1 << 4);
        break;

    case 7:
        trisbit = TRISH & (1 << 5);
        break;

    default:
        return;
    }

    TCPPutROMString( sktHTTP, trisbit ? (ROM void*)"In" : (ROM void*)"Out" );
    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_DigitalSwDisabled( WORD ch )
{
    BYTE trisbit;

    trisbit = get_digital_tris( (BYTE)ch );

    if ( 0xff != trisbit && trisbit )
    {
        TCPPutROMString( sktHTTP, (ROM void*)"disabled" );
    }
}


void HTTPPrint_DigitalPORT( WORD num, WORD act )
{
    BYTE portbit;

    if ( 0 <= num && num <= 7 )
    {
        portbit = PIO_LOW_IN & (1 << num);
    }
    else if ( 8 <= num && num <= 15 )
    {
        num -=8;
        portbit = PIO_HIGH_IN & (1 << num);
    }
    else
    {
        return;
    }

    if ( act )
    {
        TCPPutROMString( sktHTTP, portbit ? (ROM void*)"H" : (ROM void*)"L" );
    }
    else
    {
        TCPPutROMString( sktHTTP, portbit ? (ROM void*)"L" : (ROM void*)"H" );
    }
    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_LCDline( WORD line )
{
    char text[17];

    if ( 0 == line )
    {
        memcpy( (void*)text, (void*)LCDText, 16 );
    }
    else
    {
        memcpy( (void*)text, (void*)LCDText + 16, 16 );
    }

    text[16] = '\0';
    TCPPutString( sktHTTP, text );
    curHTTP.callbackPos = 0x00;
}

extern APP_CONFIG AppConfig;

void HTTPPrintIP(IP_ADDR ip)
{
    BYTE digits[4];
    BYTE i;

    for(i = 0; i < 4u; i++)
    {
        if(i)
            TCPPut(sktHTTP, '.');
        uitoa(ip.v[i], digits);
        TCPPutString(sktHTTP, digits);
    }
}

void HTTPPrint_tcp_sw( WORD num )
{
    BOOL bCheck = FALSE;

    switch ( num )
    {
    case 0:
        if ( AppConfig.Flags.bLcdPortIsTcp )
        {
            bCheck = TRUE;
        }
        break;
    case 1:
        if ( AppConfig.Flags.bParallelPortIsTcp )
        {
            bCheck = TRUE;
        }
        break;
    case 2:
        if ( AppConfig.Flags.bSerialPortIsTcp )
        {
            bCheck = TRUE;
        }
        break;
    case 3:
        if ( AppConfig.Flags.bCDCPortIsTcp )
        {
            bCheck = TRUE;
        }
        break;
    default:
        break;
    }

    if ( bCheck )
    {
        TCPPutROMString(sktHTTP, (ROM BYTE*)"checked");
    }
    else
    {
        curHTTP.callbackPos = 0x00;
    }
}

void HTTPPrint_config_dhcpchecked(void)
{
    if(AppConfig.Flags.bIsDHCPEnabled)
        TCPPutROMString(sktHTTP, (ROM BYTE*)"checked");
    return;
}

void HTTPPrint_config_ip(void)
{
    if ( AppConfig.Flags.bIsDHCPEnabled )
    {
        HTTPPrintIP(AppConfig.MyIPAddr);
    }
    else
    {
        HTTPPrintIP(AppConfig.DefaultIPAddr);
    }
    return;
}

void HTTPPrint_config_gw(void)
{
//    HTTPPrintIP(AppConfig.MyGateway);
    HTTPPrintIP(AppConfig.DefaultGateway);
    return;
}

void HTTPPrint_config_subnet(void)
{
//TS    HTTPPrintIP(AppConfig.MyMask);
    HTTPPrintIP(AppConfig.DefaultMask);
    return;
}

void HTTPPrint_config_dns1( void )
{
    HTTPPrintIP( AppConfig.PrimaryDNSServer );
    return;
}

void HTTPPrint_config_dns2( void )
{
    HTTPPrintIP( AppConfig.SecondaryDNSServer );
    return;
}


void HTTPPrint_config_mac(void)
{
    BYTE i;

    if(TCPIsPutReady(sktHTTP) < 18u)
    {//need 17 bytes to write a MAC
        curHTTP.callbackPos = 0x01;
        return;
    }

    // Write each byte
    for(i = 0; i < 6u; i++)
    {
        if(i)
            TCPPut(sktHTTP, ':');
        TCPPut(sktHTTP, btohexa_high(AppConfig.MyMACAddr.v[i]));
        TCPPut(sktHTTP, btohexa_low(AppConfig.MyMACAddr.v[i]));
    }

    // Indicate that we're done
    curHTTP.callbackPos = 0x00;
    return;
}

void HTTPPrint_config_ip_port( WORD num )
{
    char  tmp[6];
    WORD* pt;

    switch ( num )
    {
    case 0:
        pt = &AppConfig.HTTPPort;
        break;
    case 1:
        pt = &AppConfig.LCDPort;
        break;
    case 2:
        pt = &AppConfig.PARALLELPort;
        break;
    case 3:
        pt = &AppConfig.SERIALPort;
        break;
    case 4:
        pt = &AppConfig.CDCPort;
        break;
    default:
        curHTTP.callbackPos = 0x00;
        return;
    }

    if( TCPIsPutReady(sktHTTP) < 5 )
    {
        curHTTP.callbackPos = 0x01;
        return;
    }

//    sprintf( tmp, "%d", *pt );    // ver 1.36
    sprintf( tmp, "%u", *pt );      // ver 1.36
    TCPPutString( sktHTTP, tmp );
    curHTTP.callbackPos = 0x00;
}

void HTTPPrint_reboot(void)
{
    // This is not so much a print function, but causes the board to reboot
    // when the configuration is changed.  If called via an AJAX call, this
    // will gracefully reset the board and bring it back online immediately
    Reset();
}

void HTTPPrint_rebootaddr(void)
{// This is the expected address of the board upon rebooting
    TCPPutString(sktHTTP, curHTTP.data);
}

// PIC-NIC
void StringToPort( char* str, WORD* pPort )
{
    long lTemp;

    lTemp = atol( str );
    if ( 0 < lTemp && lTemp <= 65535 )
    {
        *pPort = (WORD)lTemp;
    }
}

static HTTP_IO_RESULT HTTPPostPICNICConfig(void)
{
    APP_CONFIG newAppConfig;
    BYTE *ptr;
    BYTE i;
    BOOL bInit = TRUE;
    long lTemp;

    lastFailure = TRUE;

    // Check to see if the browser is attempting to submit more data than we
    // can parse at once.  This function needs to receive all updated
    // parameters and validate them all before committing them to memory so that
    // orphaned configuration parameters do not get written (for example, if a
    // static IP address is given, but the subnet mask fails parsing, we
    // should not use the static IP address).  Everything needs to be processed
    // in a single transaction.  If this is impossible, fail and notify the user.
    // As a web devloper, if you add parameters to AppConfig and run into this
    // problem, you could fix this by to splitting your update web page into two
    // seperate web pages (causing two transactional writes).  Alternatively,
    // you could fix it by storing a static shadow copy of AppConfig someplace
    // in memory and using it instead of newAppConfig.  Lastly, you could
    // increase the TCP RX FIFO size for the HTTP server.  This will allow more
    // data to be POSTed by the web browser before hitting this limit.
    if(curHTTP.byteCount > TCPIsGetReady(sktHTTP) + TCPGetRxFIFOFree(sktHTTP))
        goto ConfigFailure;

    // Ensure that all data is waiting to be parsed.  If not, keep waiting for
    // all of it to arrive.
    if(TCPIsGetReady(sktHTTP) < curHTTP.byteCount)
        return HTTP_IO_NEED_DATA;


    // Use current config in non-volatile memory as defaults
    #if defined(EEPROM_CS_TRIS)
        XEEReadArray(0x0000, (BYTE*)&newAppConfig, sizeof(AppConfig));
    #elif defined(SPIFLASH_CS_TRIS)
        SPIFlashReadArray(0x0001, (BYTE*)&newAppConfig, sizeof(AppConfig));
    #endif

    // Start out assuming that DHCP is disabled.  This is necessary since the
    // browser doesn't submit this field if it is unchecked (meaning zero).
    // However, if it is checked, this will be overridden since it will be
    // submitted.

    newAppConfig = AppConfig;
    newAppConfig.Flags.bIsDHCPEnabled = 0;
    newAppConfig.Flags.bLcdPortIsTcp = 0;
    newAppConfig.Flags.bParallelPortIsTcp = 0;
    newAppConfig.Flags.bSerialPortIsTcp = 0;
    newAppConfig.Flags.bCDCPortIsTcp = 0;

    // Read all browser POST data
    while(curHTTP.byteCount)
    {
        // Read a form field name
        if(HTTPReadPostName(curHTTP.data, 6) != HTTP_READ_OK)
            goto ConfigFailure;

        // Read a form field value
        if(HTTPReadPostValue(curHTTP.data + 6, sizeof(curHTTP.data)-6-2) != HTTP_READ_OK)
            goto ConfigFailure;

        // Parse the value that was read
        if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"ip"))
        {// Read new static IP Address
            if(!StringToIPAddress(curHTTP.data+6, &newAppConfig.DefaultIPAddr))
                goto ConfigFailure;
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"gw"))
        {// Read new gateway address
            if(!StringToIPAddress(curHTTP.data+6, &newAppConfig.DefaultGateway))
                goto ConfigFailure;
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"sub"))
        {// Read new static subnet
            if(!StringToIPAddress(curHTTP.data+6, &newAppConfig.DefaultMask))
                goto ConfigFailure;
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"dns1"))
        {// Read new primary DNS server
            if(!StringToIPAddress(curHTTP.data+6, &newAppConfig.PrimaryDNSServer))
                goto ConfigFailure;
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"dns2"))
        {// Read new secondary DNS server
            if(!StringToIPAddress(curHTTP.data+6, &newAppConfig.SecondaryDNSServer))
                goto ConfigFailure;
        }

        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"ht"))
        {// Read http port
            StringToPort( curHTTP.data+6, &newAppConfig.HTTPPort );
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"lt"))
        {// Read LCD port
            StringToPort( curHTTP.data+6, &newAppConfig.LCDPort );
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"pu"))
        {// Read Parallel port
            StringToPort( curHTTP.data+6, &newAppConfig.PARALLELPort );
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"su"))
        {// Read Serial port
            StringToPort( curHTTP.data+6, &newAppConfig.SERIALPort );
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"cu"))
        {// Read CDC port
            StringToPort( curHTTP.data+6, &newAppConfig.CDCPort );
        }

        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"dhcp"))
        {// Read new DHCP Enabled flag
            if( 0 == strcmppgm2ram( (char*)curHTTP.data + 6, (ROM char*)"on") )
            {
                newAppConfig.Flags.bIsDHCPEnabled = 1;
            }
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"tcp1"))
        {// LCD port tcp flag
            if( 0 == strcmppgm2ram( (char*)curHTTP.data + 6, (ROM char*)"on") )
            {
                newAppConfig.Flags.bLcdPortIsTcp = 1;
            }
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"tcp2"))
        {// Parallel port tcp flag
            if( 0 == strcmppgm2ram( (char*)curHTTP.data + 6, (ROM char*)"on") )
            {
                newAppConfig.Flags.bParallelPortIsTcp = 1;
            }
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"tcp3"))
        {// Serial port tcp flag
            if( 0 == strcmppgm2ram( (char*)curHTTP.data + 6, (ROM char*)"on") )
            {
                newAppConfig.Flags.bSerialPortIsTcp = 1;
            }
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"tcp4"))
        {// CDC port tcp flag
            if( 0 == strcmppgm2ram( (char*)curHTTP.data + 6, (ROM char*)"on") )
            {
                newAppConfig.Flags.bCDCPortIsTcp = 1;
            }
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"CfSv"))
        {// Initialize Configuration Data
            bInit = FALSE;
        }
    }

    if ( !newAppConfig.Flags.bIsDHCPEnabled )
    {
        newAppConfig.MyIPAddr = newAppConfig.DefaultIPAddr;
        newAppConfig.MyMask = newAppConfig.DefaultMask;
        newAppConfig.MyGateway = newAppConfig.DefaultGateway;
    }

    if ( bInit )
    {
        // Initialize Configuration Data
        InitAppConfigSub( &AppConfig );
    }
    else
    {
        AppConfig = newAppConfig;
    }

    SaveAppConfig();

    lastFailure = FALSE;
//    Reset();  //TODO

ConfigFailure:

    strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/config.html");
    curHTTP.data[12] = 0x00;
    curHTTP.httpStatus = HTTP_REDIRECT;

    return HTTP_IO_DONE;
}

static HTTP_IO_RESULT HTTPPostPortDir(void)
{
    BYTE i;
    BOOL bInit = TRUE;
    BOOL bFound = FALSE;
    char name[6];

    lastFailure = TRUE;

    if(curHTTP.byteCount > TCPIsGetReady(sktHTTP) + TCPGetRxFIFOFree(sktHTTP))
        goto ConfigFailure;

    // Ensure that all data is waiting to be parsed.  If not, keep waiting for
    // all of it to arrive.
    if(TCPIsGetReady(sktHTTP) < curHTTP.byteCount)
        return HTTP_IO_NEED_DATA;

    strcpypgm2ram( name, (ROM void*)"pd" );

    // Read all browser POST data
    while(curHTTP.byteCount)
    {
        // Read a form field name
        if(HTTPReadPostName(curHTTP.data, 6) != HTTP_READ_OK)
            goto ConfigFailure;

        // Read a form field value
        if(HTTPReadPostValue(curHTTP.data + 6, sizeof(curHTTP.data)-6-2) != HTTP_READ_OK)
            goto ConfigFailure;

        for ( i =0; i < MAX_DIGITAL_BIT; i++ )
        {
            sprintf( name+2, "%d", i );  // "pd0" ~ "pd15"

            // Parse the value that was read
            if( 0 == strcmp( (char*)curHTTP.data, name ) )
            {
                if( 0 == strcmppgm2ram( (char*)curHTTP.data + 6, (ROM char*)"IN") )
                {
                    set_digital_tris( i, 1 );
                }
                else
                {
                    set_digital_tris( i, 0 );
                }

                if ( i == (MAX_DIGITAL_BIT - 1) )
                {
                    SaveAppConfig();
                }
                bFound = TRUE;
            }
        }

        if ( 0 == strcmppgm2ram((char*)curHTTP.data, (ROM char*)"PdSv") )
        {// Initialize Configuration Data
            bInit = FALSE;
        }
    }

    if ( bInit )
    {
        AppConfig.TRIS[0] = 0;
        AppConfig.TRIS[1] = 0;
        PIO_LOW_TRIS = AppConfig.TRIS[0];
        PIO_HIGH_TRIS = AppConfig.TRIS[1];
        SaveAppConfig();
    }

    lastFailure = FALSE;

ConfigFailure:

    strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/config.html");
    curHTTP.data[12] = 0x00;
    curHTTP.httpStatus = HTTP_REDIRECT;

    return HTTP_IO_DONE;
}

static HTTP_IO_RESULT HTTPPostPortNameSet(void)
{
    BYTE        i;
    BYTE        l;
    BOOL        bInit = TRUE;
    BOOL        bFound = FALSE;
    PORT_NAME   PortName;
    char        name[6];

    lastFailure = TRUE;

    if(curHTTP.byteCount > TCPIsGetReady(sktHTTP) + TCPGetRxFIFOFree(sktHTTP))
        goto ConfigFailure;

    // Ensure that all data is waiting to be parsed.  If not, keep waiting for
    // all of it to arrive.
    if(TCPIsGetReady(sktHTTP) < curHTTP.byteCount)
        return HTTP_IO_NEED_DATA;

    strcpypgm2ram( name, (ROM void*)"pn" );

    // Read all browser POST data
    while(curHTTP.byteCount)
    {
        // Read a form field name
        if(HTTPReadPostName(curHTTP.data, 6) != HTTP_READ_OK)
            goto ConfigFailure;

        // Read a form field value
        if(HTTPReadPostValue(curHTTP.data + 6, sizeof(curHTTP.data)-6-2) != HTTP_READ_OK)
            goto ConfigFailure;

        for ( i =0; i < MAX_IO_BIT; i++ )
        {
            sprintf( name+2, "%d", i );  // "pn0" ~ "pn23"

            // Parse the value that was read
            if( 0 == strcmp( (char*)curHTTP.data, name ) )
            {
                l = mMIN(strlen(curHTTP.data + 6), PORTNAME_LEN);
                strncpy( PortName.name, curHTTP.data + 6, l );
                PortName.name[l] = '\0';

                #if defined(EEPROM_CS_TRIS)
                    CalcSum( (BYTE*)&PortName, sizeof(PortName) - sizeof(PortName.sum), TRUE );
                    XEEBeginWrite( (WORD)PORTNAME_OFFSET + ((WORD)i * (WORD)PORTNAME_BLOCK));
                    XEEWriteArray( (BYTE*)&PortName, sizeof(PortName) );
                    DelayMs( 10 );
                #endif
                bFound = TRUE;
            }
        }

        if ( 0 == strcmppgm2ram((char*)curHTTP.data, (ROM char*)"PnSv") )
        {// Initialize Configuration Data
            bInit = FALSE;
            bFound = TRUE;
        }
    }

    if ( bInit )
    {
        bIsInitPortName = TRUE;
    }

    lastFailure = FALSE;

ConfigFailure:

    strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/config.html");
    curHTTP.data[12] = 0x00;
    curHTTP.httpStatus = HTTP_REDIRECT;

    return HTTP_IO_DONE;
}

static HTTP_IO_RESULT HTTPPostLCDSend(void)
{
    BOOL bInit = TRUE;

    lastFailure = TRUE;

    if(curHTTP.byteCount > TCPIsGetReady(sktHTTP) + TCPGetRxFIFOFree(sktHTTP))
        goto ConfigFailure;

    // Ensure that all data is waiting to be parsed.  If not, keep waiting for
    // all of it to arrive.
    if(TCPIsGetReady(sktHTTP) < curHTTP.byteCount)
        return HTTP_IO_NEED_DATA;

    // Read all browser POST data
    while(curHTTP.byteCount)
    {
        // Read a form field name
        if(HTTPReadPostName(curHTTP.data, 6) != HTTP_READ_OK)
            goto ConfigFailure;

        // Read a form field value
        if(HTTPReadPostValue(curHTTP.data + 6, sizeof(curHTTP.data)-6-2) != HTTP_READ_OK)
            goto ConfigFailure;

        // Parse the value that was read
        if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"l1"))
        {
            memset( LCDText, ' ', 16 );
            strncpy( LCDText, curHTTP.data + 6, mMIN(strlen(curHTTP.data + 6), 16) );
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"l2"))
        {
            memset( LCDText + 16, ' ', 16 );
            strncpy( LCDText + 16, curHTTP.data + 6, mMIN(strlen(curHTTP.data + 6), 16) );
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"LcSv"))
        {
            bInit = FALSE;
        }
    }

    if ( bInit )
    {
        LCDDefault( 0 );
        LCDDefault( 1 );
    }
    else
    {
        LCDUpdate();
    }
    lastFailure = FALSE;

ConfigFailure:

    strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/index.html");
    curHTTP.data[11] = 0x00;
    curHTTP.httpStatus = HTTP_REDIRECT;

    return HTTP_IO_DONE;
}

BOOL CalcSum( BYTE* data, WORD len, BOOL bWrite )
{
    BYTE sum = 0;

    for ( /**/; len > 0; len-- )
    {
        sum ^= *data++;
    }

    if ( bWrite )
    {
        data[0] = sum;
        data[1] = ~sum;
        return TRUE;
    }
    else
    {
        if ( sum == data[0] && ~sum == data[1] )
        {
            return TRUE;
        }
        else
        {
            return FALSE;
        }
    }
}

static HTTP_IO_RESULT HTTPPostAuth(void)
{
    BYTE *ptr;
    BYTE i;
    BOOL bInit = TRUE;
    AUTH_CONFIG auth;
    BYTE        current[AUTH_LEN + 1];

    lastFailure = TRUE;

    if(curHTTP.byteCount > TCPIsGetReady(sktHTTP) + TCPGetRxFIFOFree(sktHTTP))
        goto ConfigFailure;

    // Ensure that all data is waiting to be parsed.  If not, keep waiting for
    // all of it to arrive.
    if(TCPIsGetReady(sktHTTP) < curHTTP.byteCount)
        return HTTP_IO_NEED_DATA;

    AppConfig.Flags.bIsAuth = 0;
    auth.authflag = 0;

    // Read all browser POST data
    while(curHTTP.byteCount)
    {
        DEBUGLOGRS_UART( "Auth Loop\r\n" );

        // Read a form field name
        if(HTTPReadPostName(curHTTP.data, 6) != HTTP_READ_OK)
            goto ConfigFailure;

        DEBUGLOGS_UART( curHTTP.data );
        DEBUGLOGRS_UART( "\r\n" );

        // Read a form field value
        if(HTTPReadPostValue(curHTTP.data + 6, sizeof(curHTTP.data)-6-2) != HTTP_READ_OK)
            goto ConfigFailure;

        DEBUGLOGS_UART( curHTTP.data + 6 );
        DEBUGLOGRS_UART( "\r\n" );

        // Parse the value that was read
        if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"ah"))
        {// Read new Auth Enabled flag
            if( 0 == strcmppgm2ram( (char*)curHTTP.data + 6, (ROM char*)"on") )
            {
                AppConfig.Flags.bIsAuth = 1;
                auth.authflag = 1;
            }
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"ps1"))
        {
            strncpy( (char*)auth.password, (char*)curHTTP.data + 6, AUTH_LEN );
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"ps0"))
        {
            strncpy( (char*)current, (char*)curHTTP.data + 6, AUTH_LEN );
        }

        else if ( 0 == strcmppgm2ram((char*)curHTTP.data, (ROM char*)"AhSv") )
        {// Initialize Data
            bInit = FALSE;
        }
    }

    if ( bInit )
    {
        // Initialize Auth Data
        DEBUGLOGRS_UART( "INIT Auth\r\n" );
        strcpypgm2ram( (char*)auth.password, (ROM char*)MY_DEFAULT_AUTH_PASSWORD );
        AppConfig.Flags.bIsAuth = 0;
        auth.authflag = 0;
    }

    if ( 0 == strcmppgm2ram( (char*)auth.password, (ROM char *)"****" ) &&
       ( 0 == strcmppgm2ram( (char*)current, (ROM char *)"****" )) )
    {
        strncpy( (char*)auth.password, (char*)AuthConfig.password, AUTH_LEN );
        strncpy( (char*)current,       (char*)AuthConfig.password, AUTH_LEN );
    }

    DEBUGLOGRS_UART( auth.authflag ? "1" : "0" );
    DEBUGLOGRS_UART( "\r\n" );

    if ( 0 == strcmp( AuthConfig.password, current ) )
    {
        // All parsing complete!  Save new settings and force a reboot
        #if defined(EEPROM_CS_TRIS)
            DEBUGLOGRS_UART( "Save Auth\r\n" );

            CalcSum( (BYTE*)&auth, sizeof(auth) - sizeof(auth.sum), TRUE );
            XEEBeginWrite( AUTH_OFFSET );
            XEEWriteArray( (BYTE*)&auth, sizeof(auth) );
        #endif

        AuthConfig = auth;
        PasswordModified = MODIFIED_SUCCESS;
    }
    else
    {
        PasswordModified = MODIFIED_FAILED;
    }

    lastFailure = FALSE;

ConfigFailure:

    strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/config.html");
    curHTTP.data[12] = 0x00;
    curHTTP.httpStatus = HTTP_REDIRECT;

    return HTTP_IO_DONE;
}

static HTTP_IO_RESULT HTTPPostNTP(void)
{
    BYTE *ptr;
    BYTE i;
    BOOL bInit = TRUE;
    NTPSERVERNameT tempNTP;

    lastFailure = TRUE;

    if(curHTTP.byteCount > TCPIsGetReady(sktHTTP) + TCPGetRxFIFOFree(sktHTTP))
        goto ConfigFailure;

    // Ensure that all data is waiting to be parsed.  If not, keep waiting for
    // all of it to arrive.
    if(TCPIsGetReady(sktHTTP) < curHTTP.byteCount)
        return HTTP_IO_NEED_DATA;

    memset( (void*)&tempNTP, 0, sizeof(tempNTP) );

    // Read all browser POST data
    while(curHTTP.byteCount)
    {
        // Read a form field name
        if(HTTPReadPostName(curHTTP.data, 6) != HTTP_READ_OK)
            goto ConfigFailure;

        // Read a form field value
        if(HTTPReadPostValue(curHTTP.data + 6, sizeof(curHTTP.data)-6-2) != HTTP_READ_OK)
            goto ConfigFailure;

        // Parse the value that was read
        if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"ntp"))
        {
            strncpy( tempNTP.name, (char*)curHTTP.data + 6, sizeof(tempNTP.name) );
        }
        else if ( 0 == strcmppgm2ram((char*)curHTTP.data, (ROM char*)"ntSv") )
        {// Initialize Data
            bInit = FALSE;
        }
    }

    if ( bInit )
    {
        // Initialize Auth Data
        memset( (void*)&tempNTP, 0, sizeof(tempNTP) );
        strcpypgm2ram( tempNTP.name, (ROM char*)DEFAULT_NTP_SERVER );
        DEBUGLOGRS_UART( "INIT NTP\r\n" );
    }

    // All parsing complete!  Save new settings and force a reboot
    #if defined(EEPROM_CS_TRIS)
        CalcSum( (BYTE*)&tempNTP, sizeof(tempNTP) - sizeof(tempNTP.sum), TRUE );
        XEEBeginWrite( NTP_OFFSET );
        XEEWriteArray( (BYTE*)&tempNTP, sizeof(tempNTP) );
    #endif

    gNTPServerName = tempNTP;

    lastFailure = FALSE;

ConfigFailure:

    strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/config.html");
    curHTTP.data[12] = 0x00;
    curHTTP.httpStatus = HTTP_REDIRECT;

    return HTTP_IO_DONE;
}

static HTTP_IO_RESULT HTTPPostADConf(void)
{
    BYTE *ptr;
    BOOL bInit = TRUE;

    lastFailure = TRUE;

    if(curHTTP.byteCount > TCPIsGetReady(sktHTTP) + TCPGetRxFIFOFree(sktHTTP))
        goto ConfigFailure;

    // Ensure that all data is waiting to be parsed.  If not, keep waiting for
    // all of it to arrive.
    if(TCPIsGetReady(sktHTTP) < curHTTP.byteCount)
        return HTTP_IO_NEED_DATA;

    // Read all browser POST data
    while(curHTTP.byteCount)
    {
        // Read a form field name
        if(HTTPReadPostName(curHTTP.data, 6) != HTTP_READ_OK)
            goto ConfigFailure;

        // Read a form field value
        if(HTTPReadPostValue(curHTTP.data + 6, sizeof(curHTTP.data)-6-2) != HTTP_READ_OK)
            goto ConfigFailure;

        if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"vr0"))
        {
            // Parse the value that was read
            if( 0 == strcmppgm2ram( (char*)curHTTP.data + 6, (ROM char*)"IN") )
            {
                AppConfig.ADCONF.VCFG0 = 0;
            }
            else
            {
                AppConfig.ADCONF.VCFG0 = 1;
            }
        }
        else if(!strcmppgm2ram((char*)curHTTP.data, (ROM char*)"vr1"))
        {
            // Parse the value that was read
            if( 0 == strcmppgm2ram( (char*)curHTTP.data + 6, (ROM char*)"IN") )
            {
                AppConfig.ADCONF.VCFG1 = 0;
            }
            else
            {
                AppConfig.ADCONF.VCFG1 = 1;
            }
        }
        else if ( 0 == strcmppgm2ram((char*)curHTTP.data, (ROM char*)"adSv") )
        {// Initialize Data
            bInit = FALSE;
        }
    }

    if ( bInit )
    {
        // Initialize A/D Conf Data
        AppConfig.ADCONF.VCFG0 = 0;
        AppConfig.ADCONF.VCFG1 = 0;
        DEBUGLOGRS_UART( "INIT A/D Conf\r\n" );
    }

    ChangeADConf();
    SaveAppConfig();

    lastFailure = FALSE;

ConfigFailure:

    strcpypgm2ram((char*)curHTTP.data, (ROM char*)"/config.html");
    curHTTP.data[12] = 0x00;
    curHTTP.httpStatus = HTTP_REDIRECT;

    return HTTP_IO_DONE;
}
#endif
