/*********************************************************************
 *
 *  Main Application Entry Point and TCP/IP Stack Demo
 *  Module for Microchip TCP/IP Stack
 *   -Demonstrates how to call and use the Microchip TCP/IP stack
 *   -Reference: AN833
 *
 *********************************************************************
 * FileName:        MainDemo.c
 * Dependencies:    TCPIP.h
 * Processor:       PIC18, PIC24F, PIC24H, dsPIC30F, dsPIC33F, PIC32
 * Compiler:        Microchip C32 v1.05 or higher
 *                  Microchip C30 v3.12 or higher
 *                  Microchip C18 v3.30 or higher
 *                  HI-TECH PICC-18 PRO 9.63PL2 or higher
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * Copyright (C) 2002-2009 Microchip Technology Inc.  All rights
 * reserved.
 *
 * Microchip licenses to you the right to use, modify, copy, and
 * distribute:
 * (i)  the Software when embedded on a Microchip microcontroller or
 *      digital signal controller product ("Device") which is
 *      integrated into Licensee's product; or
 * (ii) ONLY the Software driver source files ENC28J60.c, ENC28J60.h,
 *      ENCX24J600.c and ENCX24J600.h ported to a non-Microchip device
 *      used in conjunction with a Microchip ethernet controller for
 *      the sole purpose of interfacing with the ethernet controller.
 *
 * You should refer to the license agreement accompanying this
 * Software for additional information regarding your rights and
 * obligations.
 *
 * THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * MICROCHIP BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE
 * THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER
 * SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.
 *
 *
 * Author              Date         Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Nilesh Rajbharti     4/19/01     Original (Rev. 1.0)
 * Nilesh Rajbharti     2/09/02     Cleanup
 * Nilesh Rajbharti     5/22/02     Rev 2.0 (See version.log for detail)
 * Nilesh Rajbharti     7/9/02      Rev 2.1 (See version.log for detail)
 * Nilesh Rajbharti     4/7/03      Rev 2.11.01 (See version log for detail)
 * Howard Schlunder     10/1/04     Beta Rev 0.9 (See version log for detail)
 * Howard Schlunder     10/8/04     Beta Rev 0.9.1 Announce support added
 * Howard Schlunder     11/29/04    Beta Rev 0.9.2 (See version log for detail)
 * Howard Schlunder     2/10/05     Rev 2.5.0
 * Howard Schlunder     1/5/06      Rev 3.00
 * Howard Schlunder     1/18/06     Rev 3.01 ENC28J60 fixes to TCP,
 *                                  UDP and ENC28J60 files
 * Howard Schlunder     3/01/06     Rev. 3.16 including 16-bit micro support
 * Howard Schlunder     4/12/06     Rev. 3.50 added LCD for Explorer 16
 * Howard Schlunder     6/19/06     Rev. 3.60 finished dsPIC30F support, added PICDEM.net 2 support
 * Howard Schlunder     8/02/06     Rev. 3.75 added beta DNS, NBNS, and HTTP client (GenericTCPClient.c) services
 * Howard Schlunder     12/28/06    Rev. 4.00RC added SMTP, Telnet, substantially modified TCP layer
 * Howard Schlunder     04/09/07    Rev. 4.02 added TCPPerformanceTest, UDPPerformanceTest, Reboot and fixed some bugs
 * Howard Schlunder     xx/xx/07    Rev. 4.03
 * HSchlunder & EWood   08/27/07    Rev. 4.11
 * HSchlunder & EWood   10/08/07    Rev. 4.13
 * HSchlunder & EWood   11/06/07    Rev. 4.16
 * HSchlunder & EWood   11/08/07    Rev. 4.17
 * HSchlunder & EWood   11/12/07    Rev. 4.18
 * HSchlunder & EWood   02/11/08    Rev. 4.19
 * HSchlunder & EWood   04/26/08    Rev. 4.50 Moved most code to other files for clarity
 * KHesky               07/07/08    Added ZG2100-specific support
 * HSchlunder & EWood   07/24/08    Rev. 4.51
 * Howard Schlunder     11/10/08    Rev. 4.55
 * Howard Schlunder     04/14/09    Rev. 5.00
 * Howard Schlunder     07/10/09    Rev. 5.10
 ********************************************************************/
/*
 * This macro uniquely defines this file as the main entry point.
 * There should only be one such definition in the entire project,
 * and this file must define the AppConfig variable as described below.
 */
#define THIS_IS_STACK_APPLICATION

// Include all headers for any enabled TCPIP Stack functions
#include "TCPIP Stack/TCPIP.h"

// Include functions specific to this stack application
#include "MainDemo.h"
#include "MicroWire.h" //TS
#include "usb_config.h"
#include "./USB/usb.h"
#include "./USB/usb_function_cdc.h"

#include "PICNICTask.h"
void SIOTask( int mode );
void PIOTask( void );
void LCDTask( void );
void InitPIOTask( void );
void InitSIOTask( void );
void InitLCDTask( void );
void InitializeUSBSystem(void);

extern BYTE ENCRevID;

extern BOOL CalcSum( BYTE* data, WORD len, BOOL bWrite );
extern void putsBOOT( const char* str );
extern void WriteBOOT( BYTE chr );
extern BOOL BusyBOOT( void );


// Declare AppConfig structure and some other supporting stack variables
APP_CONFIG AppConfig;
AUTH_CONFIG AuthConfig;
//BYTE AN0String[8];

/* ver 1.36 Serial Interrupt */
BYTE SIORingBuffer[SIO_BUF_SIZE];
BYTE SIOwritep;
BYTE SIOreadp;
BYTE SIOrxcount;

// Use UART2 instead of UART1 for stdout (printf functions).  Explorer 16
// serial port hardware is on PIC UART2 module.
#if defined(EXPLORER_16)
    int __C30_UART = 2;
#endif


// Private helper functions.
// These may or may not be present in all applications.
static void InitAppConfig(void);
static void InitializeBoard(void);
static void ProcessIO(void);

// ver 1.36 Serial Interrupt
static void vSerialRxISR( void );

//
// PIC18 Interrupt Service Routines
//
// NOTE: Several PICs, including the PIC18F4620 revision A3 have a RETFIE FAST/MOVFF bug
// The interruptlow keyword is used to work around the bug when using C18
#if defined(__18CXX)
    #if defined(HI_TECH_C)
    void interrupt low_priority LowISR(void)
    #else
    #pragma interruptlow LowISR
    void LowISR(void)
    #endif
    {
        TickUpdate();
    }

    #if defined(HI_TECH_C)
    void interrupt HighISR(void)
    #else
    #pragma interruptlow HighISR
    void HighISR(void)
    #endif
    {
        // ver1.36 Serial
        if( PIR1bits.RCIF )
        {
            vSerialRxISR();
        }
        #if defined(STACK_USE_UART2TCP_BRIDGE)
//            UART2TCPBridgeISR();
        #endif
    }

    #if !defined(HI_TECH_C)
//    #pragma code lowVector=0x18
//    void LowVector(void){_asm goto LowISR _endasm}
//    #pragma code highVector=0x8
//    void HighVector(void){_asm goto HighISR _endasm}
//    #pragma code // Return to default code section

#if 1 //

#define UBW
/** V E C T O R  R E M A P P I N G **********************************/
#ifdef UBW // UBW
extern void _startup (void);        // See c018i.c in your C18 compiler dir
#pragma code _RESET_INTERRUPT_VECTOR = 0x001000
void _reset (void)
{
    _asm goto _startup _endasm
}
#pragma code high_vector=0x01008
void interrupt_at_high_vector(void)
{
    _asm goto HighISR _endasm
}
#pragma code _LOW_INTERRUPT_VECTOR = 0x001018
void _low_ISR (void)
{
    _asm goto LowISR _endasm
}
#endif
#pragma code
#endif //

    #endif

// C30 and C32 Exception Handlers
// If your code gets here, you either tried to read or write
// a NULL pointer, or your application overflowed the stack
// by having too many local variables or parameters declared.
#elif defined(__C30__)
    void _ISR __attribute__((__no_auto_psv__)) _AddressError(void)
    {
        Nop();
        Nop();
    }
    void _ISR __attribute__((__no_auto_psv__)) _StackError(void)
    {
        Nop();
        Nop();
    }

#elif defined(__C32__)
    void _general_exception_handler(unsigned cause, unsigned status)
    {
        Nop();
        Nop();
    }
#endif

//
// Main application entry point.
//
#if defined(__18CXX)
void main(void)
#else
int main(void)
#endif
{
    static BOOL bLCDupdate = FALSE;
    static TICK t = 0;
    static DWORD dwLastIP = 0;
    DWORD ntptime = 0;
    DWORD ntptemp;
    static DWORD last_ntptime = 0;
    long y, m, d, days;
    WORD hour, sec, min;

    InitializeUSBSystem();

    // Initialize application specific hardware
    InitializeBoard();

    #if defined(USE_LCD)
    // Initialize and display the stack version on the LCD
    LCDInit();

    #endif

    // Initialize stack-related hardware components that may be
    // required by the UART configuration routines
    TickInit();
    #if defined(STACK_USE_MPFS) || defined(STACK_USE_MPFS2)
    MPFSInit();
    #endif

    // Initialize Stack and application related NV variables into AppConfig.
    InitAppConfig();

    InitPIOTask();
    InitSIOTask();
    InitLCDTask();

    // Initiates board setup process if button is depressed
    // on startup

    if ( 0 == PORTBbits.RB4 )
    {
        // Start Bootstrap mode
        LED_PUT( 0xff );

        #if defined(STACK_USE_UART)
        sprintf( &LCDText[ 0], "Bootstrap mode  " );
        sprintf( &LCDText[16], "    Wait Rx Data" );
        LCDUpdate();
        DoUARTConfig();
        #endif
    }

    // Initialize core stack layers (MAC, ARP, TCP, UDP) and
    // application modules (HTTP, SNMP, etc.)
    StackInit();

    sprintf( &LCDText[ 0], "PIC-NIC Ver%d.%d", PICNIC_VER1, PICNIC_VER2 );
    sprintf( &LCDText[16], "ENC28J60 Rev.%d", ENCRevID );
    LCDUpdate();
    DelayMs( 2000 );

    LCDDefault( 1 );

    // Now that all items are initialized, begin the co-operative
    // multitasking loop.  This infinite loop will continuously
    // execute all stack-related tasks, as well as your own
    // application's functions.  Custom functions should be added
    // at the end of this loop.
    // Note that this is a "co-operative mult-tasking" mechanism
    // where every task performs its tasks (whether all in one shot
    // or part of it) and returns so that other tasks can do their
    // job.
    // If a task needs very long time to do its job, it must be broken
    // down into smaller pieces so that other tasks can have CPU time.
    while(1)
    {
        // Blink LED0 (right most one) every second.
        if(TickGet() - t >= TICK_SECOND/2ul)
        {
            t = TickGet();
            LED0_IO ^= 1;
        }

        // Check bus status and service USB interrupts.
        USBDeviceTasks(); // Interrupt or polling method.  If using polling, must call
                          // this function periodically.  This function will take care
                          // of processing and responding to SETUP transactions
                          // (such as during the enumeration process when you first
                          // plug in).  USB hosts require that USB devices should accept
                          // and process SETUP packets in a timely fashion.  Therefore,
                          // when using polling, this function should be called
                          // frequently (such as once about every 100 microseconds) at any
                          // time that a SETUP packet might reasonably be expected to
                          // be sent by the host to your device.  In most cases, the
                          // USBDeviceTasks() function does not take very long to
                          // execute (~50 instruction cycles) before it returns.

        // This task performs normal stack task including checking
        // for incoming packet, type of packet and calling
        // appropriate stack entity to process it.
        StackTask();

        // This tasks invokes each of the core stack application tasks
        StackApplications();

        // Process application specific tasks here.
        // For this demo app, this will include the Generic TCP
        // client and servers, and the SNMP, Ping, and SNMP Trap
        // demos.  Following that, we will process any IO from
        // the inputs on the board itself.
        // Any custom modules or processing you need to do should
        // go here.
        #if defined(STACK_USE_GENERIC_TCP_CLIENT_EXAMPLE)
        //GenericTCPClient();
        #endif

        #if defined(STACK_USE_GENERIC_TCP_SERVER_EXAMPLE)
        //GenericTCPServer();
        #endif

        #if defined(STACK_USE_SMTP_CLIENT)
        //SMTPDemo();
        #endif

        #if defined(STACK_USE_ICMP_CLIENT)
//TS        PingDemo();
        #endif

        #if defined(STACK_USE_SNMP_SERVER) && !defined(SNMP_TRAP_DISABLED)
        //SNMPTrapDemo();
        if(gSendTrapFlag)
        //    SNMPSendTrap();
        #endif

        #if defined(STACK_USE_BERKELEY_API)
        //BerkeleyTCPClientDemo();
        //BerkeleyTCPServerDemo();
        //BerkeleyUDPClientDemo();
        #endif

        ProcessIO();

        // If the local IP address has changed (ex: due to DHCP lease change)
        // write the new IP address to the LCD display, UART, and Announce
        // service
        if(dwLastIP != AppConfig.MyIPAddr.Val)
        {
            dwLastIP = AppConfig.MyIPAddr.Val;

            #if defined(STACK_USE_UART)
                DEBUGLOGRS_UART((ROM char*)"\r\nNew IP Address: ");
            #endif

            LCDDefault( 0 );

            #if defined(STACK_USE_UART)
                DEBUGLOGRS_UART((ROM char*)"\r\n");
            #endif

            #if defined(STACK_USE_ANNOUNCE)
                AnnounceIP();
            #endif
        }

        LCDTask();
        PIOTask();
        SIOTask( MODE_SIO );
        SIOTask( MODE_CDC );
#if 1
        //See http://www2.nict.go.jp/w/w114/tsp/PubNtp/clients.html
        ntptime = SNTPGetUTCSeconds();

        if ( 0 == ntptime || last_ntptime == ntptime )
        {
            continue;
        }

        last_ntptime = ntptime;

        if( BUTTON0_IO != 0u )
        {
            if ( bLCDupdate )
            {
                LCDDefault( 0 );
                LCDDefault( 1 );
                bLCDupdate = FALSE;
            }
            continue;
        }

        ntptime += (3600ul * 9ul); // +0900(JST)

        // C18timen̕W֐(localtime/ctime/asctime)̂ŁAOŌvZ
        days = (ntptime / 86400) + (1970ul * 365ul + 113ul);
        ntptemp = ntptime % 86400;

        hour = ntptemp / 3600;
        ntptemp %= 3600;

        min = ntptemp / 60;
        ntptemp %= 60;

        sec = ntptemp;

        y = (days + 305) / 146097 * 400
            + (days + 305) % 146097 / 36524 * 100
            + (days + 305) % 146097 % 36524 / 1461 * 4
            + (days + 305) % 146097 % 36524 % 1461 / 365;
        d = days - (365 * (y - 1) + y / 4 - y / 100 + y / 400 + 31 + 28 + 1);

        if( d < 0 )
        {
            y--;
            d = 365;
        }

        for( m = 3; m < 15; m++ )
        {
            if( 306 * (m+1) / 10 - 122 <= d && 306 * (m + 2) / 10 - 122 > d )
                break;
        }

        d -= 306 * (m + 1) / 10 - 122 - 1;

        if(m > 12)
        {
            y++;
            m -= 12;
        }

        sprintf( &LCDText[0],  " %02d:%02d:%02d (JST) ", hour, min, sec );
        sprintf( &LCDText[16], "%04ld-%02ld-%02ld (",y, m, d );

        switch ( days % 7 )
        {
        case 0 :
            sprintf( &LCDText[28], "Sun)" );
            break;
        case 1 :
            sprintf( &LCDText[28], "Mon)" );
            break;
        case 2 :
            sprintf( &LCDText[28], "Tue)" );
            break;
        case 3 :
            sprintf( &LCDText[28], "Wed)" );
            break;
        case 4 :
            sprintf( &LCDText[28], "Thu)" );
            break;
        case 5 :
            sprintf( &LCDText[28], "Fri)" );
            break;
        case 6 :
        default:
            sprintf( &LCDText[28], "Sat)" );
            break;
        }

        LCDUpdate();
        bLCDupdate = TRUE;
#endif
    }
}

void LCDDefault( BYTE num )
{
    if ( 0 == num )
    {
        memset( (void*)&LCDText[0], ' ', 16 );
        if ( AppConfig.Flags.bIsDHCPEnabled )
        {
            LCDText[0] = '*';
        }
        else
        {
            LCDText[0] = ' ';
        }
        DisplayIPValue( AppConfig.MyIPAddr, 0 );
    }
    else
    {
        memset( (void*)&LCDText[16], ' ', 16 );
//        strcpypgm2ram( (char*)&LCDText[16], "TCPStack " VERSION );
        strcpypgm2ram( (char*)&LCDText[16], "TriState PIC-NIC" );
        LCDUpdate();
    }
}

// Writes an IP address to the LCD display and the UART as available
void DisplayIPValue( IP_ADDR IPVal, int num )
{
//  printf("%u.%u.%u.%u", IPVal.v[0], IPVal.v[1], IPVal.v[2], IPVal.v[3]);
    BYTE IPDigit[4];
    BYTE i;
#ifdef USE_LCD
    BYTE j;
//    BYTE LCDPos=0;
    BYTE LCDPos=1;
#endif

    for(i = 0; i < sizeof(IP_ADDR); i++)
    {
        uitoa((WORD)IPVal.v[i], IPDigit);

        #if defined(STACK_USE_UART)
            if ( 0 == num )
            {
//                putsUART( IPDigit );
            }
            else
            {
//                putsBOOT( IPDigit );
            }
        #endif

        #ifdef USE_LCD
        if ( 0 == num )
        {
            for(j = 0; j < strlen((char*)IPDigit); j++)
            {
                LCDText[LCDPos++] = IPDigit[j];
            }
            if(i == sizeof(IP_ADDR)-1)
                break;
            LCDText[LCDPos++] = '.';
        }
        else
        {
            if(i == sizeof(IP_ADDR)-1)
                break;
        }
        #else
            if(i == sizeof(IP_ADDR)-1)
                break;
        #endif

        #if defined(STACK_USE_UART)
            if ( 0 == num )
            {
//                while(BusyUART());
//                WriteUART('.');
            }
            else
            {
//                while(BusyBOOT());
//                WriteBOOT('.');
            }
        #endif
    }

    #ifdef USE_LCD
    if ( 0 == num )
    {
        if(LCDPos < 32u)
            LCDText[LCDPos] = 0;
        LCDUpdate();
    }
    #endif
}

// Processes A/D data from the potentiometer
static void ProcessIO(void)
{
    char tmp;
    // AN0 should already be set up as an analog input
//TS    ADCON0bits.GO = 1;

    // Wait until A/D conversion is done
//TS    while(ADCON0bits.GO);

    // Blink LED (USB Status)
    //BlinkUSBStatus();

    // AD converter errata work around (ex: PIC18F87J10 A2)
//TS    #if !defined(__18F87J50) && !defined(_18F87J50) && !defined(__18F87J11) && !defined(_18F87J11)
//        PRODL = ADCON2;
//        ADCON2 |= 0x7;  // Select Frc mode by setting ADCS0/ADCS1/ADCS2
//        ADCON2 = PRODL;
//    #endif

    // Convert 10-bit value into ASCII string
//TS    uitoa(*((WORD*)(&ADRESL)), AN0String);

    // USB CDC Service
    if( (USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1) )
    {
        LED1_IO = 1;
        return;
    }

    LED1_IO = 0;
    CDCTxService();
}

/****************************************************************************
  Function:
    static void InitializeBoard(void)

  Description:
    This routine initializes the hardware.  It is a generic initialization
    routine for many of the Microchip development boards, using definitions
    in HardwareProfile.h to determine specific initialization.

  Precondition:
    None

  Parameters:
    None - None

  Returns:
    None

  Remarks:
    None
  ***************************************************************************/
static void InitializeBoard(void)
{
    // LEDs
    LED0_TRIS = 0;
    LED1_TRIS = 0;
    LED2_TRIS = 0;
    LED3_TRIS = 0;
    LED4_TRIS = 0;
    LED5_TRIS = 0;
    LED6_TRIS = 0;

#if !defined(EXPLORER_16)   // Pin multiplexed with a button on EXPLORER_16
    LED7_TRIS = 0;
#endif
    LED_PUT(0x00);

#if defined(__18CXX)
    // Enable 4x/5x/96MHz PLL on PIC18F87J10, PIC18F97J60, PIC18F87J50, etc.
//    OSCTUNE = 0x40;

    // Set up analog features of PORTA

    // PICDEM.net 2 board has POT on AN2, Temp Sensor on AN3
    #if defined(PICDEMNET2)
        ADCON0 = 0x09;      // ADON, Channel 2
        ADCON1 = 0x0B;      // Vdd/Vss is +/-REF, AN0, AN1, AN2, AN3 are analog
    #elif defined(PICDEMZ)
        ADCON0 = 0x81;      // ADON, Channel 0, Fosc/32
        ADCON1 = 0x0F;      // Vdd/Vss is +/-REF, AN0, AN1, AN2, AN3 are all digital
    #elif defined(__18F87J11) || defined(_18F87J11) || defined(__18F87J50) || defined(_18F87J50)
        ADCON0 = 0x01;      // ADON, Channel 0, Vdd/Vss is +/-REF
        WDTCONbits.ADSHR = 1;
//TS        ANCON0 = 0xFC;      // AN0 (POT) and AN1 (temp sensor) are anlog
        ADCON1 = 0x38;      //20TAD
        ANCON0 = 0x00;      // AN0~AN4,AN7 are anlog (for PIC-NIC)
        ANCON1 = 0xC0;      // AN10~AN13 are anlog (for PIC-NIC)
        WDTCONbits.ADSHR = 0;
    #else
        ADCON0 = 0x01;      // ADON, Channel 0
        ADCON1 = 0x0E;      // Vdd/Vss is +/-REF, AN0 is analog
    #endif
    ADCON2 = 0xBE;      // Right justify, 20TAD ACQ time, Fosc/64 (~21.0kHz)

    // Enable internal PORTB pull-ups
    INTCON2bits.RBPU = 0;

    // Configure USART

    SIOrxcount = 0;
    SIOwritep = 0;
    SIOreadp = 0;

    TXSTA = 0x20;
    RCSTA = 0x90;

    SetSIOBaudRate( BAUD_RATE );

/*
    // See if we can use the high baud rate setting
    #if ((GetPeripheralClock()+2*BAUD_RATE)/BAUD_RATE/4 - 1) <= 255
        SPBRG = (GetPeripheralClock()+2*BAUD_RATE)/BAUD_RATE/4 - 1;
        TXSTAbits.BRGH = 1;
    #else   // Use the low baud rate setting
        SPBRG = (GetPeripheralClock()+8*BAUD_RATE)/BAUD_RATE/16 - 1;
    #endif
*/

    // Serial Interrupt
    IPR1bits.RCIP = 1;      // Priority High
    PIE1bits.RCIE = 1;      // Enable Interrupt

    // Enable Interrupts
    RCONbits.IPEN = 1;      // Enable interrupt priorities
    INTCONbits.GIEH = 1;
    INTCONbits.GIEL = 1;

    // Do a calibration A/D conversion
    #if defined(__18F87J10) || defined(__18F86J15) || defined(__18F86J10) || defined(__18F85J15) || defined(__18F85J10) || defined(__18F67J10) || defined(__18F66J15) || defined(__18F66J10) || defined(__18F65J15) || defined(__18F65J10) || defined(__18F97J60) || defined(__18F96J65) || defined(__18F96J60) || defined(__18F87J60) || defined(__18F86J65) || defined(__18F86J60) || defined(__18F67J60) || defined(__18F66J65) || defined(__18F66J60) || \
         defined(_18F87J10) ||  defined(_18F86J15) || defined(_18F86J10)  ||  defined(_18F85J15) ||  defined(_18F85J10) ||  defined(_18F67J10) ||  defined(_18F66J15) ||  defined(_18F66J10) ||  defined(_18F65J15) ||  defined(_18F65J10) ||  defined(_18F97J60) ||  defined(_18F96J65) ||  defined(_18F96J60) ||  defined(_18F87J60) ||  defined(_18F86J65) ||  defined(_18F86J60) ||  defined(_18F67J60) ||  defined(_18F66J65) ||  defined(_18F66J60)
        ADCON0bits.ADCAL = 1;
        ADCON0bits.GO = 1;
        while(ADCON0bits.GO);
        ADCON0bits.ADCAL = 0;
    #elif defined(__18F87J11) || defined(__18F86J16) || defined(__18F86J11) || defined(__18F67J11) || defined(__18F66J16) || defined(__18F66J11) || \
           defined(_18F87J11) ||  defined(_18F86J16) ||  defined(_18F86J11) ||  defined(_18F67J11) ||  defined(_18F66J16) ||  defined(_18F66J11) || \
          defined(__18F87J50) || defined(__18F86J55) || defined(__18F86J50) || defined(__18F67J50) || defined(__18F66J55) || defined(__18F66J50) || \
           defined(_18F87J50) ||  defined(_18F86J55) ||  defined(_18F86J50) ||  defined(_18F67J50) ||  defined(_18F66J55) ||  defined(_18F66J50)
        ADCON1bits.ADCAL = 1;
        ADCON0bits.GO = 1;
        while(ADCON0bits.GO);
        ADCON1bits.ADCAL = 0;
    #endif

#else   // 16-bit C30 and and 32-bit C32
    #if defined(__PIC32MX__)
    {
        // Enable multi-vectored interrupts
        INTEnableSystemMultiVectoredInt();

        // Enable optimal performance
        SYSTEMConfigPerformance(GetSystemClock());
        mOSCSetPBDIV(OSC_PB_DIV_1);             // Use 1:1 CPU Core:Peripheral clocks

        // Disable JTAG port so we get our I/O pins back, but first
        // wait 50ms so if you want to reprogram the part with
        // JTAG, you'll still have a tiny window before JTAG goes away
        DelayMs(50);
        DDPCONbits.JTAGEN = 0;
        LED_PUT(0x00);              // Turn the LEDs off
    }
    #endif

    #if defined(__dsPIC33F__) || defined(__PIC24H__)
        // Crank up the core frequency
        PLLFBD = 38;                // Multiply by 40 for 160MHz VCO output (8MHz XT oscillator)
        CLKDIV = 0x0000;            // FRC: divide by 2, PLLPOST: divide by 2, PLLPRE: divide by 2

        // Port I/O
        AD1PCFGHbits.PCFG23 = 1;    // Make RA7 (BUTTON1) a digital input
        AD1PCFGHbits.PCFG20 = 1;    // Make RA12 (INT1) a digital input for ZeroG ZG2100M PICtail Plus interrupt

        // ADC
        AD1CHS0 = 0;                // Input to AN0 (potentiometer)
        AD1PCFGLbits.PCFG5 = 0;     // Disable digital input on AN5 (potentiometer)
        AD1PCFGLbits.PCFG4 = 0;     // Disable digital input on AN4 (TC1047A temp sensor)
    #else   //defined(__PIC24F__) || defined(__PIC32MX__)
        #if defined(__PIC24F__)
            CLKDIVbits.RCDIV = 0;       // Set 1:1 8MHz FRC postscalar
        #endif

        // ADC
        AD1CHS = 0;                 // Input to AN0 (potentiometer)
        AD1PCFGbits.PCFG4 = 0;      // Disable digital input on AN4 (TC1047A temp sensor)
        #if defined(__32MX460F512L__)   // PIC32MX460F512L PIM has different pinout to accomodate USB module
            AD1PCFGbits.PCFG2 = 0;      // Disable digital input on AN2 (potentiometer)
        #else
            AD1PCFGbits.PCFG5 = 0;      // Disable digital input on AN5 (potentiometer)
        #endif
    #endif

    // ADC
    AD1CON1 = 0x84E4;           // Turn on, auto sample start, auto-convert, 12 bit mode (on parts with a 12bit A/D)
    AD1CON2 = 0x0404;           // AVdd, AVss, int every 2 conversions, MUXA only, scan
    AD1CON3 = 0x1003;           // 16 Tad auto-sample, Tad = 3*Tcy
    #if defined(__32MX460F512L__)   // PIC32MX460F512L PIM has different pinout to accomodate USB module
        AD1CSSL = 1<<2;             // Scan pot
    #else
        AD1CSSL = 1<<5;             // Scan pot
    #endif

    // UART
    #if defined(STACK_USE_UART)
        UARTTX_TRIS = 0;
        UARTRX_TRIS = 1;
        UMODE = 0x8000;         // Set UARTEN.  Note: this must be done before setting UTXEN

        #if defined(__C30__)
            USTA = 0x0400;      // UTXEN set
            #define CLOSEST_UBRG_VALUE ((GetPeripheralClock()+8ul*BAUD_RATE)/16/BAUD_RATE-1)
            #define BAUD_ACTUAL (GetPeripheralClock()/16/(CLOSEST_UBRG_VALUE+1))
        #else   //defined(__C32__)
            USTA = 0x00001400;      // RXEN set, TXEN set
            #define CLOSEST_UBRG_VALUE ((GetPeripheralClock()+8ul*BAUD_RATE)/16/BAUD_RATE-1)
            #define BAUD_ACTUAL (GetPeripheralClock()/16/(CLOSEST_UBRG_VALUE+1))
        #endif

        #define BAUD_ERROR ((BAUD_ACTUAL > BAUD_RATE) ? BAUD_ACTUAL-BAUD_RATE : BAUD_RATE-BAUD_ACTUAL)
        #define BAUD_ERROR_PRECENT  ((BAUD_ERROR*100+BAUD_RATE/2)/BAUD_RATE)
        #if (BAUD_ERROR_PRECENT > 3)
            #warning UART frequency error is worse than 3%
        #elif (BAUD_ERROR_PRECENT > 2)
            #warning UART frequency error is worse than 2%
        #endif

        UBRG = CLOSEST_UBRG_VALUE;
    #endif

#endif

#if defined(EEPROM_CS_TRIS)
    XEEInit();
#endif

   TRISBbits.TRISB4 = 1;
}

// 09.12.26 Tristate
void SetSIOBaudRate( DWORD baud_rate )
{
    // See if we can use the high baud rate setting
    if ( ((GetPeripheralClock()+2*baud_rate)/baud_rate/4 - 1) <= 255 )
    {
        SPBRG = (GetPeripheralClock()+2*baud_rate)/baud_rate/4 - 1;
        TXSTAbits.BRGH = 1;
    }
    else   // Use the low baud rate setting
    {
        SPBRG = (GetPeripheralClock()+8*baud_rate)/baud_rate/16 - 1;
        TXSTAbits.BRGH = 0;
    }
}

/*********************************************************************
 * Function:        void InitAppConfig(void)
 *
 * PreCondition:    MPFSInit() is already called.
 *
 * Input:           None
 *
 * Output:          Write/Read non-volatile config variables.
 *
 * Side Effects:    None
 *
 * Overview:        None
 *
 * Note:            None
 ********************************************************************/
// MAC Address Serialization using a MPLAB PM3 Programmer and
// Serialized Quick Turn Programming (SQTP).
// The advantage of using SQTP for programming the MAC Address is it
// allows you to auto-increment the MAC address without recompiling
// the code for each unit.  To use SQTP, the MAC address must be fixed
// at a specific location in program memory.  Uncomment these two pragmas
// that locate the MAC address at 0x1FFF0.  Syntax below is for MPLAB C
// Compiler for PIC18 MCUs. Syntax will vary for other compilers.
//#pragma romdata MACROM=0x1FFF0
//static ROM BYTE SerializedMACAddress[6] = {MY_DEFAULT_MAC_BYTE1, MY_DEFAULT_MAC_BYTE2, MY_DEFAULT_MAC_BYTE3, MY_DEFAULT_MAC_BYTE4, MY_DEFAULT_MAC_BYTE5, MY_DEFAULT_MAC_BYTE6};
//#pragma romdata

void InitAppConfigSub( APP_CONFIG* p_config )
{
    p_config->Flags.bIsDHCPEnabled = 1;
    p_config->Flags.bInConfigMode = 1;
    p_config->Flags.bLcdPortIsTcp = 0;
    p_config->Flags.bParallelPortIsTcp = 0;
    p_config->Flags.bSerialPortIsTcp = 0;
    p_config->Flags.bCDCPortIsTcp = 0;

    p_config->MyIPAddr.Val = MY_DEFAULT_IP_ADDR_BYTE1 | MY_DEFAULT_IP_ADDR_BYTE2<<8ul | MY_DEFAULT_IP_ADDR_BYTE3<<16ul | MY_DEFAULT_IP_ADDR_BYTE4<<24ul;
    p_config->DefaultIPAddr.Val = AppConfig.MyIPAddr.Val;

    p_config->MyMask.Val = MY_DEFAULT_MASK_BYTE1 | MY_DEFAULT_MASK_BYTE2<<8ul | MY_DEFAULT_MASK_BYTE3<<16ul | MY_DEFAULT_MASK_BYTE4<<24ul;
    p_config->DefaultMask.Val = AppConfig.MyMask.Val;

    p_config->MyGateway.Val = MY_DEFAULT_GATE_BYTE1 | MY_DEFAULT_GATE_BYTE2<<8ul | MY_DEFAULT_GATE_BYTE3<<16ul | MY_DEFAULT_GATE_BYTE4<<24ul;
    p_config->DefaultGateway.Val = AppConfig.MyGateway.Val;

    p_config->PrimaryDNSServer.Val = MY_DEFAULT_PRIMARY_DNS_BYTE1 | MY_DEFAULT_PRIMARY_DNS_BYTE2<<8ul  | MY_DEFAULT_PRIMARY_DNS_BYTE3<<16ul  | MY_DEFAULT_PRIMARY_DNS_BYTE4<<24ul;
    p_config->SecondaryDNSServer.Val = MY_DEFAULT_SECONDARY_DNS_BYTE1 | MY_DEFAULT_SECONDARY_DNS_BYTE2<<8ul  | MY_DEFAULT_SECONDARY_DNS_BYTE3<<16ul  | MY_DEFAULT_SECONDARY_DNS_BYTE4<<24ul;

    p_config->HTTPPort = MY_DEFAULT_HTTP_PORT;
    p_config->LCDPort = MY_DEFAULT_LCD_PORT;
    p_config->PARALLELPort = MY_DEFAULT_PARALLEL_PORT;
    p_config->SERIALPort = MY_DEFAULT_SERIAL_PORT;
    p_config->CDCPort = MY_DEFAULT_CDC_PORT;

    p_config->ADCONF.VCFG1 = 0; //AVSS
    p_config->ADCONF.VCFG0 = 0; //AVDD
}

static void InitAppConfig(void)
{
    // Read MacAddress(PIC-NIC)
    BYTE mac[6];
    InitMicroWire();
    ReadMacAddress( mac );
    // <---

    memset( &AppConfig, 0, sizeof(AppConfig) );

    InitAppConfigSub( &AppConfig );

//P.P    memcpypgm2ram((void*)&AppConfig.MyMACAddr, (ROM void*)SerializedMACAddress, sizeof(AppConfig.MyMACAddr));
    memcpy( (void*)&AppConfig.MyMACAddr, (void*)mac, sizeof(AppConfig.MyMACAddr));

//  {
//      _prog_addressT MACAddressAddress;
//      MACAddressAddress.next = 0x157F8;
//      _memcpy_p2d24((char*)&AppConfig.MyMACAddr, MACAddressAddress, sizeof(AppConfig.MyMACAddr));
//  }

    // SNMP Community String configuration
    #if defined(STACK_USE_SNMP_SERVER)
    {
        BYTE i;
        static ROM char * ROM cReadCommunities[] = SNMP_READ_COMMUNITIES;
        static ROM char * ROM cWriteCommunities[] = SNMP_WRITE_COMMUNITIES;
        ROM char * strCommunity;

        for(i = 0; i < SNMP_MAX_COMMUNITY_SUPPORT; i++)
        {
            // Get a pointer to the next community string
            strCommunity = cReadCommunities[i];
            if(i >= sizeof(cReadCommunities)/sizeof(cReadCommunities[0]))
                strCommunity = "";

            // Ensure we don't buffer overflow.  If your code gets stuck here,
            // it means your SNMP_COMMUNITY_MAX_LEN definition in TCPIPConfig.h
            // is either too small or one of your community string lengths
            // (SNMP_READ_COMMUNITIES) are too large.  Fix either.
            if(strlenpgm(strCommunity) >= sizeof(AppConfig.readCommunity[0]))
                while(1);

            // Copy string into AppConfig
            strcpypgm2ram((char*)AppConfig.readCommunity[i], strCommunity);

            // Get a pointer to the next community string
            strCommunity = cWriteCommunities[i];
            if(i >= sizeof(cWriteCommunities)/sizeof(cWriteCommunities[0]))
                strCommunity = "";

            // Ensure we don't buffer overflow.  If your code gets stuck here,
            // it means your SNMP_COMMUNITY_MAX_LEN definition in TCPIPConfig.h
            // is either too small or one of your community string lengths
            // (SNMP_WRITE_COMMUNITIES) are too large.  Fix either.
            if(strlenpgm(strCommunity) >= sizeof(AppConfig.writeCommunity[0]))
                while(1);

            // Copy string into AppConfig
            strcpypgm2ram((char*)AppConfig.writeCommunity[i], strCommunity);
        }
    }
    #endif

#if 0 //Tristate
    // Load the default NetBIOS Host Name
    memcpypgm2ram(AppConfig.NetBIOSName, (ROM void*)MY_DEFAULT_HOST_NAME, 16);
    FormatNetBIOSName(AppConfig.NetBIOSName);
#endif

    #if defined(EEPROM_CS_TRIS)
    {
        APP_CONFIG newAppConfig;
        XEEReadArray( CONFIG_OFFSET, (BYTE*)&newAppConfig, sizeof(newAppConfig) );
        if ( !CalcSum( (BYTE*)&newAppConfig, sizeof(newAppConfig) - sizeof(newAppConfig.sum), FALSE ) )
        {
            SaveAppConfig();
        }
        else
        {
            AppConfig = newAppConfig;

            // Copy IP Address(Manual)
            if ( !AppConfig.Flags.bIsDHCPEnabled )
                AppConfig.MyIPAddr.Val = AppConfig.DefaultIPAddr.Val;

            DEBUGLOGRS_UART( "READ AppConfig\r\n" );
        }

        // Read Auth Password
        XEEReadArray( AUTH_OFFSET, (BYTE*)&AuthConfig, sizeof(AuthConfig) );

        if ( !CalcSum( AuthConfig.password, sizeof(AuthConfig) - sizeof(AuthConfig.sum), FALSE ) )
        {
            DEBUGLOGRS_UART( "Set Default Password\r\n" );
            strcpypgm2ram( (char*)AuthConfig.password, (ROM char*)MY_DEFAULT_AUTH_PASSWORD );
            AuthConfig.authflag = 0;
            AppConfig.Flags.bIsAuth = 0;
        }
        else
        {
            DEBUGLOGRS_UART( "Read Password\r\n" );
            DEBUGLOGRS_UART( "\r\n" );
            AppConfig.Flags.bIsAuth = AuthConfig.authflag;
        }

        // Read NTP Server Address
        XEEReadArray( NTP_OFFSET, (BYTE*)&gNTPServerName, sizeof(gNTPServerName) );
        if ( !CalcSum( (BYTE*)&gNTPServerName, sizeof(gNTPServerName) - sizeof(gNTPServerName.sum), FALSE ) )
        {
            DEBUGLOGRS_UART( "Set Default NTP\r\n" );
            strcpypgm2ram( (char*)&gNTPServerName.name, (ROM char*)DEFAULT_NTP_SERVER );
        }
        else
        {
            DEBUGLOGRS_UART( "Read NTP\r\n" );
        }

   }
    #endif

    PIO_LOW_TRIS = AppConfig.TRIS[0];
    PIO_HIGH_TRIS = AppConfig.TRIS[1];
    PIO_LOW_OUT = 0x00;
    PIO_HIGH_OUT = 0x00;

    ChangeADConf();
}

#if defined(EEPROM_CS_TRIS) || defined(SPIFLASH_CS_TRIS)
void SaveAppConfig(void)
{
    // Ensure adequate space has been reserved in non-volatile storage to
    // store the entire AppConfig structure.  If you get stuck in this while(1)
    // trap, it means you have a design time misconfiguration in TCPIPConfig.h.
    // You must increase MPFS_RESERVE_BLOCK to allocate more space.
    #if defined(STACK_USE_MPFS) || defined(STACK_USE_MPFS2)
        if(sizeof(AppConfig) > MPFS_RESERVE_BLOCK)
            while(1);
    #endif

    #if defined(EEPROM_CS_TRIS)
        CalcSum( (BYTE*)&AppConfig, sizeof(AppConfig) - sizeof(AppConfig.sum), TRUE );
        XEEBeginWrite( CONFIG_OFFSET );
        XEEWriteArray( (BYTE*)&AppConfig, sizeof(AppConfig) );
        DEBUGLOGRS_UART( "SAVE AppConfig\r\n" );
    #endif
}
#endif

void putrsUART(const rom char* a)
{
    putrsUSART( a );
}

void ChangeADConf( void )
{
    ADCON0bits.GO = 0;
    ADCON0bits.VCFG0 = AppConfig.ADCONF.VCFG0;
    ADCON0bits.VCFG1 = AppConfig.ADCONF.VCFG1;
}

/* ver 1.36 */

/**
    SIOM`FbNiOobt@MJEgL`FbNj
*/
BOOL DataRdySIO( void )
{
    return ( SIOrxcount > 0 ) ? TRUE : FALSE;
}

/**
    SIOPMiOobt@̂Poj
*/
BYTE ReadSIO( void )
{
    BYTE rxdata;

    if ( SIOrxcount > 0 )
    {
        rxdata = SIORingBuffer[SIOreadp++];

        if ( SIOreadp >= SIO_BUF_SIZE )
            SIOreadp = 0;

        SIOrxcount--;
    }
    else
    {
        rxdata = 0;
    }

    return rxdata;
}

/**
    SIOM荞
*/
static void vSerialRxISR( void )
{
    BYTE rxData;

    rxData = RCREG;

    /* Clear any overrun errors. */

    if( RCSTAbits.OERR )
    {
        // Clear Overrun Error
        RCSTAbits.CREN = 0;
        RCSTAbits.CREN = 1;
    }

    if ( SIOrxcount < SIO_BUF_SIZE )
    {
        SIORingBuffer[SIOwritep++] = rxData;
        if ( SIOwritep >= SIO_BUF_SIZE )
        {
            SIOwritep = 0;
        }
        SIOrxcount++;
    }
}

