#define __PICNIC_TASK_C

#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "TCPIPConfig.h"
#include "TCPIP Stack/TCPIP.h"
#include "PICNICTask.h"
#include "MainDemo.h"

static UDP_PORT g_udp_port = 4096u;

BYTE PhaseOpen( tRemoteInfo* pInfo )
{
    BYTE result = PHASE_ERROR;

    if ( !pInfo->flags.bOpened )
    {
        switch ( pInfo->route )
        {
        case ROUTE_IS_UDP:
            pInfo->socket.udp = UDPOpen( pInfo->port, NULL, g_udp_port++ );
            if ( INVALID_UDP_SOCKET != pInfo->socket.udp )
            {
                pInfo->flags.bOpened = TRUE;
                result = PHASE_IDLE;
            }
            break;

        case ROUTE_IS_TCP:
            DEBUGLOGRS_UART( "Open\r\n" );
            pInfo->socket.tcp = TCPOpen( NULL, TCP_OPEN_SERVER, pInfo->port, TCP_PURPOSE_DEFAULT );

            if ( INVALID_SOCKET != pInfo->socket.tcp )
            {
                TCPWasReset( pInfo->socket.tcp );
                pInfo->flags.bOpened = TRUE;
                result = PHASE_LISTEN;
            }
            break;

        case ROUTE_IS_SERIAL:
            pInfo->flags.bOpened = TRUE;
            break;

        case ROUTE_IS_CDC:
            break;

        default:
            break;
        }
    }
    return result;
}

BYTE PhaseListen( tRemoteInfo* pInfo )
{
    BYTE result = PHASE_ERROR;

    if ( pInfo->flags.bOpened )
    {
        switch ( pInfo->route )
        {
        case ROUTE_IS_UDP:
            result = PHASE_IDLE;
            break;

        case ROUTE_IS_TCP:
            TCPWasReset( pInfo->socket.tcp );
            if ( TCPIsConnected( pInfo->socket.tcp ) )
            {
                DEBUGLOGRS_UART( "Connect\r\n" );
                result = PHASE_IDLE;
            }
            else
            {
                result = PHASE_LISTEN;
            }
            break;

        case ROUTE_IS_SERIAL:
        case ROUTE_IS_CDC:
            break;

        default:
            break;
        }
    }
    return result;
}

BYTE PhaseClose( tRemoteInfo* pInfo )
{
    BYTE result = PHASE_ERROR;

    if ( pInfo->flags.bOpened )
    {
        switch ( pInfo->route )
        {
        case ROUTE_IS_UDP:
            UDPClose( pInfo->socket.udp );
            pInfo->socket.udp = INVALID_UDP_SOCKET;
            pInfo->flags.bOpened = FALSE;
            result = PHASE_OPEN;
            break;

        case ROUTE_IS_TCP:
            DEBUGLOGRS_UART( "Close\r\n" );
            TCPClose( pInfo->socket.tcp );
            pInfo->socket.tcp = INVALID_SOCKET;
            pInfo->flags.bOpened = FALSE;
            result = PHASE_OPEN;
            break;

        case ROUTE_IS_SERIAL:
        case ROUTE_IS_CDC:
            break;

        default:
            break;
        }
    }
    return result;
}


BOOL GetByteFromNetwork( tRemoteInfo* pInfo, BYTE* data )
{
    BOOL result = FALSE;

    if ( pInfo->flags.bOpened )
    {
        switch ( pInfo->route )
        {
        case ROUTE_IS_UDP:
            if ( 0 < UDPIsGetReady( pInfo->socket.udp ) )
            {
                result = UDPGet( data );
            }
            break;

        case ROUTE_IS_TCP:
            if ( 0 < TCPIsGetReady( pInfo->socket.tcp ) )
            {
                result = TCPGet( pInfo->socket.tcp, data );
            }
            break;

        case ROUTE_IS_SERIAL:
//            if ( DataRdyUART() )
            if ( DataRdySIO() )
            {
//                result = ReadUART();
                result = ReadSIO();
            }
            break;

        case ROUTE_IS_CDC:
            break;

        default:
            break;
        }
    }

    return result;
}

WORD GetArrayFromNetwork( tRemoteInfo* pInfo, BYTE* data, WORD wDataLen )
{
    WORD result = 0;
    WORD tmp;
    WORD t1;
    BYTE len = 0;

    if ( pInfo->flags.bOpened )
    {
        switch ( pInfo->route )
        {
        case ROUTE_IS_UDP:
            if ( UDPIsGetReady( pInfo->socket.udp ) )
            {
                while ( UDPGet( data++ ) )
                {
                    if ( ++result >= wDataLen )
                    {
                        break;
                    }
                }
            }
            break;

        case ROUTE_IS_TCP:
            for ( t1 = 0; t1 < 100; t1++ )
            {
                if ( wDataLen >= TCPIsGetReady( pInfo->socket.tcp ) )
                {
                    break;
                }
                StackTask();
            }
            result = TCPGetArray( pInfo->socket.tcp, data, wDataLen );
            break;

        case ROUTE_IS_SERIAL:
            break;

        case ROUTE_IS_CDC:
            break;

        default:
            break;
        }
    }

    return result;
}

BOOL PutArrayToNetwork( tRemoteInfo* pInfo, BYTE* data, WORD length )
{
    BOOL result = FALSE;

    if ( pInfo->flags.bOpened )
    {
        switch ( pInfo->route )
        {
        case ROUTE_IS_UDP:
            while ( length > UDPIsPutReady( pInfo->socket.udp ) )
            {
                StackTask();
            }
            UDPPutArray( data, length );
            UDPFlush();
            break;

        case ROUTE_IS_TCP:
            while ( length > TCPIsPutReady( pInfo->socket.tcp ) )
            {
                StackTask();
            }
            TCPPutArray( pInfo->socket.tcp, data, length );
            TCPFlush( pInfo->socket.tcp );
            break;

        case ROUTE_IS_SERIAL:
        case ROUTE_IS_CDC:
            break;

        default:
            break;
        }
    }
    return result;
}

BOOL PutStringToNetwork( tRemoteInfo* pInfo, char* string )
{
    BOOL result = FALSE;
    WORD len;

    if ( pInfo->flags.bOpened )
    {
        len = strlen( string );

        switch ( pInfo->route )
        {
        case ROUTE_IS_UDP:
            while ( len > UDPIsPutReady( pInfo->socket.udp ) )
            {
                StackTask();
            }
            UDPPutString( string );
            UDPFlush();
            break;

        case ROUTE_IS_TCP:
            while ( len > TCPIsPutReady( pInfo->socket.tcp ) )
            {
                StackTask();
            }
            TCPPutString( pInfo->socket.tcp, string );
            TCPFlush( pInfo->socket.tcp );
            break;

        case ROUTE_IS_SERIAL:
            DEBUGLOGS_UART( string );
            break;

        case ROUTE_IS_CDC:
            break;

        default:
            break;
        }
    }
    return result;
}
