#ifndef __PICNIC_TASK_H__
#define __PICNIC_TASK_H__

#define SIO_GET_STATUS      0x00
#define SIO_REGISTER        0x01
#define SIO_UNREGISTER      0x02
#define SIO_TRANSMIT        0x03
#define SIO_RECEIVE         0x04

#define SIO_BAUD_9600       129
#define SIO_BAUD_19200      64
#define SIO_BAUD_38400      32
#define SIO_BAUD_57600      21
#define SIO_BAUD_115200     10
#define SIO_BAUD_230400     5

#define PIO_GET_STATUS      0x00
#define PIO_SET_HIGH        0x01
#define PIO_SET_INPUT       0x01
#define PIO_SET_LOW         0x02
#define PIO_SET_OUTPUT      0x02
#define PIO_SET_WHOLE       0x03
#define PIO_GET_ADC         0x04

#define LCD_PUTSTRING       0x00
#define LCD_LCDCONTROL      0x01

#define PORT_RJ             0x05
#define PORT_RE             0x06
#define TRIS_RJ             0x85
#define TRIS_RE             0x86

//#define TELNET_PORT         23

#define LCD_DHCP_FLAG       (1<<0)
#define PIO_DHCP_FLAG       (1<<1)
#define SIO_DHCP_FLAG       (1<<2)
#define CDC_DHCP_FLAG       (1<<3)

enum
{
    ROUTE_IS_TCP = 0,       // TCP
    ROUTE_IS_UDP,           // UDP
    ROUTE_IS_SERIAL,        // RS-232C
    ROUTE_IS_CDC            // USB(CDC)
};

enum
{
    MODE_PIO = 0,           // Parallel
    MODE_SIO,               // Serial
    MODE_CDC,               // CDC
    MODE_LCD                // LCD
};

typedef struct tagRemoteInfo
{
    struct
    {
        BYTE bInitialized : 1;
        BYTE bRegistered : 1;
        BYTE bAscMode : 1;
        BYTE bOpened : 1;
        BYTE stateNum : 4;
    }flags;

    WORD        port;       // TCP/UDP Port Number
    BYTE        route;      // TCP or UDP
    BYTE        mode;       // PIO, SIO, CDC, LCD

    union
    {
        struct
        {
            BYTE    mode;
        }lcd;

        struct
        {
            BYTE    mode;
        }pio;

        struct
        {
            BYTE    baud;
            BYTE    mode;
        }sio;
    }info;

    union
    {
        TCP_SOCKET  tcp;    // TCP Socket
        UDP_SOCKET  udp;    // UDP Socket
    }socket;

    WORD bufferNum; // UDP buffer Number
} tRemoteInfo;

static enum
{
    SM_PRINT = 0,
    SM_COMMAND
} TASK_STATE;

typedef enum
{
    PHASE_OPEN = 0,
    PHASE_LISTEN,
    PHASE_IDLE,
    PHASE_CLOSE,
    PHASE_ERROR
} PHASE_NUMBER;

BYTE PhaseOpen( tRemoteInfo* pInfo );
BYTE PhaseListen( tRemoteInfo* pInfo );
BYTE PhaseClose( tRemoteInfo* pInfo );
BOOL GetByteFromNetwork( tRemoteInfo* pInfo, BYTE* data );
BOOL PutStringToNetwork( tRemoteInfo* pInfo, char* string );
BOOL PutArrayToNetwork( tRemoteInfo* pInfo, BYTE* data, WORD length );
WORD GetArrayFromNetwork( tRemoteInfo* pInfo, BYTE* data, WORD wDataLen );

#endif  //__PICNIC_TASK_H__
