#define __PIOTASK_C

#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "TCPIPConfig.h"
#include "TCPIP Stack/TCPIP.h"
#include "PICNICTask.h"
#include "MainDemo.h"

void PIOresponce( tRemoteInfo* pInfo, BOOL bAD, BYTE ADch, BYTE wait );

static BYTE PhaseGetData( tRemoteInfo* pInfo );
static void pioCommand( tRemoteInfo* pInfo, BYTE* cmdbuf );

extern BOOL gDHCPReceive;

static tRemoteInfo info;

#define Delay5us(count)    Delay10TCYx(((GetInstructionClock()/2000000)*(count)))

void InitPIOTask( void )
{
    memset( &info, 0x00, sizeof(tRemoteInfo) );
}

void PIOTask( void )
{
    if ( !info.flags.bInitialized )
    {
        info.port = AppConfig.PARALLELPort;
        info.mode = MODE_PIO;
        info.flags.stateNum = PHASE_OPEN;

        if ( AppConfig.Flags.bParallelPortIsTcp )
        {
            info.route = ROUTE_IS_TCP;
        }
        else
        {
            info.route = ROUTE_IS_UDP;
        }
/*
        if ( ROUTE_IS_TCP == info.route && TELNET_PORT == info.port )
        {
            info.flags.bAscMode = 1;
        }
*/
        info.flags.bInitialized = 1;
    }

    if ( gDHCPReceive & PIO_DHCP_FLAG )
    {
        info.flags.stateNum = PHASE_CLOSE;
        gDHCPReceive &= ~PIO_DHCP_FLAG;
    }

    switch ( info.flags.stateNum )
    {
    case PHASE_OPEN:
        info.flags.stateNum = PhaseOpen( &info );
        break;
    case PHASE_LISTEN:
        info.flags.stateNum = PhaseListen( &info );
        break;
    case PHASE_CLOSE:
        info.flags.stateNum = PhaseClose( &info );
        break;
    case PHASE_IDLE:
        PhaseGetData( &info );
        break;
    default:
        break;
    }
}

BYTE getRATris( void )
{
    BYTE result;

    result  = TRISA & 0x03;             // AN0,1
    result |= (TRISAbits.TRISA4 << 2);  // AN4
    result |= (TRISFbits.TRISF2 << 3);  // AN7
    result |= (TRISFbits.TRISF5 << 4);  // AN10
    result |= (TRISFbits.TRISF6 << 5);  // AN11
    result |= (TRISHbits.TRISH4 << 6);  // AN12
    result |= (TRISHbits.TRISH5 << 7);  // AN13

    return result;
}

WORD GetAnalog( BYTE ch, BYTE wait )
{
    char str[13];

    ADCON0 &= 0xC1;  // CHS0-3 = 0

    switch ( ch )
    {
    case 0:
    case 0x81:  //CH0
        break;
    case 1:
    case 0x89:  //CH1
        ADCON0 |= (1 << 2);
        break;
    case 2:
    case 0x91:  //CH2
        ADCON0 |= (4 << 2);
        break;
    case 3:
    case 0x99:  //CH3
        ADCON0 |= (7 << 2);
        break;
    case 4:
    case 0xa1:  //CH4
        ADCON0 |= (10 << 2);
        break;
    case 5:
    case 0xa9:  //CH5
        ADCON0 |= (11 << 2);
        break;
    case 6:
    case 0xb1:  //CH6
        ADCON0 |= (12 << 2);
        break;
    case 7:
    case 0xb9:  //CH7
        ADCON0 |= (13 << 2);
        break;
    default:
        return 0;
    }

    Delay5us( wait );
    ADCON0bits.GO = 1;
    while( ADCON0bits.GO )
        ;

    return (WORD)ADRES;
}

void setHigh( BYTE port, BYTE bit )
{
    BYTE data;

    BYTE tmp[8];

    DEBUGLOGRS_UART( "SET_HIGH\r\n" );

    sprintf( tmp, "%02X", port );
    DEBUGLOGS_UART( tmp );
    DEBUGLOGRS_UART( "\r\n" );
    sprintf( tmp, "%02X", bit );
    DEBUGLOGS_UART( tmp );
    DEBUGLOGRS_UART( "\r\n" );

    if ( 0 <= bit && bit <= 7 )
    {
        data = 1 << bit;

        switch ( port )
        {
        case PORT_RE:
            PIO_LOW_OUT |= data;
            break;
        case PORT_RJ:
            PIO_HIGH_OUT |= data;
            break;
        case TRIS_RE:
            PIO_LOW_TRIS |= data;
            break;
        case TRIS_RJ:
            PIO_HIGH_TRIS |= data;
            break;
        default:
            break;
        }


    }
}

void setLow( BYTE port, BYTE bit )
{
    BYTE  mask;
    BYTE tmp[8];

    DEBUGLOGRS_UART( "SET_LOW\r\n" );

    sprintf( tmp, "%02X", port );
    DEBUGLOGS_UART( tmp );
    DEBUGLOGRS_UART( "\r\n" );
    sprintf( tmp, "%02X", bit );
    DEBUGLOGS_UART( tmp );
    DEBUGLOGRS_UART( "\r\n" );

    if ( 0 <= bit && bit <= 7 )
    {
        mask = ~(1 << bit);

        switch ( port )
        {
        case PORT_RE:
            PIO_LOW_OUT &= mask;
            break;
        case PORT_RJ:
            PIO_HIGH_OUT &= mask;
            break;
        case TRIS_RE:
            PIO_LOW_TRIS &= mask;
            break;
        case TRIS_RJ:
            PIO_HIGH_TRIS &= mask;
            break;
        default:
            break;
        }
    }
}

#define MAX_SIZE    4

void setWhole( BYTE port, BYTE size, BYTE* maskAndValue )
{
    const BYTE cPort[] = { PORT_RE, PORT_RJ, TRIS_RE, TRIS_RJ };
    BYTE data = 0;
    BYTE i;
    BYTE tmp[6];
    BYTE n;

    DEBUGLOGRS_UART( "SET_WHOLE\r\n" );

    sprintf( tmp, "%02X", port );
    DEBUGLOGS_UART( tmp );
    DEBUGLOGRS_UART( "\r\n" );
    sprintf( tmp, "%d", size );
    DEBUGLOGS_UART( tmp );
    DEBUGLOGRS_UART( "\r\n" );
    DEBUGLOGRS_UART( "\r\n" );

    if ( 0 == size || MAX_SIZE < size )
    {
        DEBUGLOGRS_UART( "Invalid sizes\r\n" );
        return;
    }

    for ( i = 0; i < size; i++ )
    {
        switch ( port )
        {
        case PORT_RE:
            data = PIO_LOW_OUT;
            n = 0;
            break;
        case PORT_RJ:
            data = PIO_HIGH_OUT;
            n = 1;
            break;
        case TRIS_RE:
            data = PIO_LOW_TRIS;
            n = 2;
            break;
        case TRIS_RJ:
            data = PIO_HIGH_TRIS;
            n = 3;
            break;
        default:
            DEBUGLOGRS_UART( "Invalid port\r\n" );
            return;
        }

        data &= *maskAndValue++;    //Mask
        data |= *maskAndValue++;    //Value

        switch ( port )
        {
        case PORT_RE:
            PIO_LOW_OUT = data;
            break;
        case PORT_RJ:
            PIO_HIGH_OUT = data;
            break;
        case TRIS_RE:
            PIO_LOW_TRIS = data;
            break;
        case TRIS_RJ:
            PIO_HIGH_TRIS = data;
            break;
        default:
            break;
        }

        n = (n+1) % MAX_SIZE;
        port = cPort[n]; // ̃|[g
    }
}

void PIOresponce( tRemoteInfo* pInfo, BOOL bAD, BYTE ADch, BYTE wait )
{
    BYTE res[8];

    res[0] = PIO_HIGH_IN;
    res[1] = PIO_LOW_IN;
    res[2] = PIO_HIGH_TRIS;
    res[3] = PIO_LOW_TRIS;

    if ( bAD )
    {
        WORD an;
        an = GetAnalog( ADch, wait );
        res[4] = an >> 8;
        res[5] = an & 0xff;
    }
    else
    {
        res[4] = 0;
        res[5] = 0;
    }
    res[6] = 0;
    res[7] = 0;
    PutArrayToNetwork( pInfo, res, 8 );
}

void pioCommand( tRemoteInfo* pInfo, BYTE* cmdbuf )
{
    BYTE res[30];
    WORD an;

    switch ( cmdbuf[0] )
    {
        case PIO_GET_STATUS:
            if ( pInfo->flags.bAscMode )
            {
                sprintf( res, "%02X, %02X, %02X, %02X, 00, 00, 00, 00\r\n", PIO_HIGH_IN, PIO_LOW_IN, PIO_HIGH_TRIS, PIO_LOW_TRIS );
                PutStringToNetwork( pInfo, res );
            }
            else
            {
                PIOresponce( pInfo, FALSE, 0, 0 );
            }
            break;
        case PIO_SET_HIGH:
            if ( pInfo->flags.bAscMode )
            {
            }
            else
            {
                setHigh( cmdbuf[1], cmdbuf[2] );
                PIOresponce( pInfo, FALSE, 0, 0 );
            }
            break;
        case PIO_SET_LOW:
            if ( pInfo->flags.bAscMode )
            {
            }
            else
            {
                setLow( cmdbuf[1], cmdbuf[2] );
                PIOresponce( pInfo, FALSE, 0, 0 );
            }
            break;
        case PIO_SET_WHOLE:
            if ( pInfo->flags.bAscMode )
            {
            }
            else
            {
                setWhole( cmdbuf[1], cmdbuf[2], cmdbuf + 3 );
                PIOresponce( pInfo, FALSE, 0, 0 );
            }
            break;
        case PIO_GET_ADC:
            if ( pInfo->flags.bAscMode )
            {
            }
            else
            {
                PIOresponce( pInfo, TRUE, cmdbuf[1], cmdbuf[2] );
            }
            break;
        default:
            break;
    }
}

static BYTE PhaseGetData( tRemoteInfo* pInfo )
{
    BYTE tmp;
    BYTE data;
    BYTE result = PHASE_IDLE;
    static BYTE pos = 0;
    static BYTE cmd = 0;
    static BYTE count = 0;
    static BYTE cmdbuf[20];

    if ( GetByteFromNetwork( pInfo, &data ) )
    {
        cmdbuf[0] = data;

        switch ( data )
        {
            case PIO_GET_STATUS:
                break;
            case PIO_SET_HIGH:
            case PIO_SET_LOW:
            case PIO_GET_ADC:
            case PIO_SET_WHOLE:
                if ( 2 < GetArrayFromNetwork( pInfo, cmdbuf + 1, 2 ) )
                {
                    return PHASE_IDLE;
                }
                else // len = 2
                {
                    if ( data == PIO_SET_WHOLE )
                    {
                        tmp = cmdbuf[2] << 1;
                        if ( tmp < GetArrayFromNetwork( pInfo, cmdbuf + 3, tmp ) )
                        {
                            return PHASE_IDLE;
                        }
                    }
                }
                break;
            default:
                return PHASE_ERROR;
        }

        if ( !pInfo->flags.bAscMode )
        {
            pioCommand( pInfo, cmdbuf );
        }
        else
        {
            cmd <<= 4;
            ++count;

            data = toupper( data );

            if ( isdigit( data ) )
            {
                data &= 0x0f;
            }
            else if ( isalnum( data ) )
            {
                data -= ('A' - 10);
            }
            else if ( '\n' == data )
            {
                pioCommand( pInfo, cmdbuf );
                pos = 0;
            }
            else
            {
                data = 0;
            }

            cmd |= data;

            if ( 2 == count )
            {
                if ( pos < sizeof(cmdbuf) )
                {
                    cmdbuf[pos++] = data;
                }
                count = 0;
            }
        }
    }

    return result;
}
