#define __SIOTASK_C

#include <string.h>
#include <stdlib.h>
#include <ctype.h>
//#include "TCPIPConfig.h"
#include "TCPIP Stack/TCPIP.h"
#include "usb_config.h"
#include "./USB/usb.h"
#include "./USB/usb_function_cdc.h"
#include "MainDemo.h"
#include "PICNICTask.h"

#define MAX_RXLEN       (100)
#define MIN_RES_SIZE    (16)
#define MAX_RES_SIZE    (MIN_RES_SIZE + MAX_RXLEN)

enum
{
    NUM_SIO = 0,
    NUM_CDC
};

extern BOOL gDHCPReceive;

static void sioCommand( tRemoteInfo* pInfo, BYTE* cmdbuf );
static void responce( tRemoteInfo* pInfo, BYTE code );
static void PhaseGetData( tRemoteInfo* pInfo );
static void initRegInfo( void );

static tRemoteInfo info[2];

static MAC_ADDR regist_mac;
static IP_ADDR regist_ip;
static WORD regist_port;
static BYTE rx_size = 0;
static BYTE recv_len = 0;
static BYTE ser_rxbuffer[MAX_RXLEN];
static BYTE res_pack[MIN_RES_SIZE + MAX_RXLEN];

void InitSIOTask( void )
{
    memset( &info[NUM_SIO], 0x00, sizeof(tRemoteInfo) );  // for SIO
    memset( &info[NUM_CDC], 0x00, sizeof(tRemoteInfo) );  // for CDC
    initRegInfo();
}

static void initRegInfo( void )
{
    memset( &regist_mac, 0x00, sizeof(regist_mac) );
    memset( &regist_ip, 0x00, sizeof(regist_ip) );
    regist_port = 0;
}

void SIOTask( int mode )
{
    tRemoteInfo* pInfo;
    BYTE         flag;

    if ( MODE_SIO == mode )
    {
        pInfo = &info[NUM_SIO];
    }
    else
    {
        pInfo = &info[NUM_CDC];
    }

    if ( !pInfo->flags.bInitialized )
    {
        if ( MODE_SIO == mode )
        {
            pInfo->port = AppConfig.SERIALPort;
            pInfo->mode = MODE_SIO;

            if ( AppConfig.Flags.bSerialPortIsTcp )
            {
                pInfo->route = ROUTE_IS_TCP;
            }
            else
            {
                pInfo->route = ROUTE_IS_UDP;
            }
        }
        else
        {
            pInfo->port = AppConfig.CDCPort;
            pInfo->mode = MODE_CDC;

            if ( AppConfig.Flags.bCDCPortIsTcp )
            {
                pInfo->route = ROUTE_IS_TCP;
            }
            else
            {
                pInfo->route = ROUTE_IS_UDP;
            }
        }

        pInfo->flags.stateNum = PHASE_OPEN;
/*
        if ( ROUTE_IS_TCP == pInfo->route && TELNET_PORT == pInfo->port )
        {
            pInfo->flags.bAscMode = 1;
        }
*/
        pInfo->flags.bInitialized = 1;
    }

    if ( MODE_SIO == mode )
    {
        flag = SIO_DHCP_FLAG;
    }
    else
    {
        flag = CDC_DHCP_FLAG;
    }

    if ( gDHCPReceive & flag )
    {
        pInfo->flags.stateNum = PHASE_CLOSE;
        gDHCPReceive &= ~flag;
    }

    switch ( pInfo->flags.stateNum )
    {
    case PHASE_OPEN:
        pInfo->flags.stateNum = PhaseOpen( pInfo );
        break;
    case PHASE_LISTEN:
        pInfo->flags.stateNum = PhaseListen( pInfo );
        break;
    case PHASE_CLOSE:
        pInfo->flags.stateNum = PhaseClose( pInfo );
        break;
    case PHASE_IDLE:
        PhaseGetData( pInfo );
        break;
    default:
        break;
    }
}

static void responce( tRemoteInfo* pInfo, BYTE code )
{
    BYTE i;
    BYTE j;
    BYTE *p;
    BYTE datasize = 0;

    for ( i = 0, j= 2; i < 6; i++ )
    {
        res_pack[j++] = regist_mac.v[i];     //MAC Address
    }

    for ( i = 0; i < 4; i++ )
    {
        res_pack[j++] = regist_ip.v[i];      //IP Address
    }

    res_pack[12] = regist_port >> 8;         //Port
    res_pack[13] = regist_port & 0xff;

    res_pack[14] = pInfo->info.sio.baud;     //Baud
    res_pack[15] = pInfo->info.sio.mode;     //Mode

    if ( MODE_SIO == pInfo->mode )
    {
        // SERIAL
        datasize = rx_size;
        memcpy( (void*)&res_pack[MIN_RES_SIZE], (void*)ser_rxbuffer, datasize );
        rx_size = 0;
    }
    else
    {
        // CDC
        CDCTxService();
        datasize = getsUSBUSART( &res_pack[MIN_RES_SIZE], MAX_RXLEN );
    }

    res_pack[ 0] = code;
    res_pack[ 1] = MIN_RES_SIZE + datasize;

    if ( SIO_RECEIVE == code && 0 == datasize )
    {
        return;
    }

    PutArrayToNetwork( pInfo, res_pack, MIN_RES_SIZE + datasize );
}

void sioCommand( tRemoteInfo* pInfo, BYTE* cmdbuf )
{
    BYTE i;
    BYTE pt;
    WORD t1;
    BYTE reg;
    BYTE data;
    DWORD baud_rate;
    SOCKET_INFO* pSInfo;

    if ( SIO_REGISTER != cmdbuf[0] && !pInfo->flags.bRegistered )
    {
        // o^ς݂łȂreturn
        DEBUGLOGRS_UART( "UNREGISTER\r\n" );
        return;
    }

    switch ( cmdbuf[0] )
    {
        case SIO_GET_STATUS:
            break;
        case SIO_REGISTER:

            if ( pInfo->flags.bRegistered )
            {
                // o^ς݂Ȃreturn
                DEBUGLOGRS_UART( "Already REGISTERED\r\n" );
                return;
            }

            // Save IP Address
            for ( i = 0; i < 4; i++ )
            {
                regist_ip.v[i] = cmdbuf[1 + i];
            }

            memset( (void*)&regist_mac, 0, sizeof(regist_mac) );

            // Get MAC Address (ARP)
            ARPResolve( &regist_ip );
            reg = 0;
            for ( t1 = 0; t1 < 1000; t1++ )
            {
                if( ARPIsResolved( &regist_ip, &regist_mac ) )
                {
                    reg = 1;
                    break;
                }

                StackTask();
            }

            if ( 0 == reg )
            {
                DEBUGLOGRS_UART( "ARP FAILED\r\n" );
                return;
            }

            if ( ROUTE_IS_UDP == pInfo->route )
            {
                for ( i = 0; UDPSocketInfo[i].localPort != pInfo->port; /**/ )
                {
                    i++;
                }

                regist_port = UDPSocketInfo[i].remotePort;
            }
            else
            {
                pSInfo = TCPGetRemoteInfo( pInfo->socket.tcp );
                regist_port = pSInfo->remotePort.Val;
            }

            DEBUGLOGRS_UART( "REGISTER\r\n" );
            pInfo->info.sio.baud = cmdbuf[7];
            pInfo->info.sio.mode = cmdbuf[8];
            pInfo->flags.bRegistered = 1;

            switch ( pInfo->info.sio.baud )
            {
            case SIO_BAUD_19200:
                baud_rate = 19200;
                break;
            case SIO_BAUD_38400:
                baud_rate = 38400;
                break;
            case SIO_BAUD_57600:
                baud_rate = 57600;
                break;
            case SIO_BAUD_115200:
                baud_rate = 115200;
                break;
            case SIO_BAUD_230400:
                baud_rate = 230400;
                break;
            case SIO_BAUD_9600:
            default:
                baud_rate = 9600;
                break;
            }

            if ( MODE_SIO == pInfo->mode )
            {
                SetSIOBaudRate( baud_rate );
            }
            else
            {
                CDCSetBaudRate( baud_rate );
            }

            break;
        case SIO_UNREGISTER:
            pInfo->flags.bRegistered = 0;
            initRegInfo();
            break;
        case SIO_TRANSMIT:
            pt = 1;

            if ( MODE_SIO == pInfo->mode )
            {
                while ( recv_len )
                {
                    // SERIAL
                    for ( t1 =0; t1 < 1000; t1++ )
                    {
                        if( !BusyUART() )
                        {
                            putcUART( cmdbuf[pt++] );
                            break;
                        }
                    }
                    recv_len--;
                }

                while ( GetArrayFromNetwork( pInfo, &data, 1 ) )
                {
                    // SERIAL
                    for ( t1 =0; t1 < 1000; t1++ )
                    {
                        if( !BusyUART() )
                        {
                            putcUART( data );
                            break;
                        }
                    }
                }
            }
            else
            {
                // CDC
                if( (USBDeviceState < CONFIGURED_STATE) || (USBSuspendControl==1) )
                {
                    // can't USB device service
                    break;
                }
                else
                {
                    // CDC
                    for ( t1 =0; t1 < 1000; t1++ )
                    {
                        USBDeviceTasks();
                        CDCTxService();
                        if ( mUSBUSARTIsTxTrfReady() )
                        {
                            putUSBUSART( &cmdbuf[pt], recv_len );

                            do {
                                USBDeviceTasks();
                                CDCTxService();
                            }while ( !USBUSARTIsTxTrfReady() );
                            break;
                        }
                    }
                }

                while ( GetArrayFromNetwork( pInfo, &data, 1 ) )
                {
                    // CDC
                    for ( t1 =0; t1 < 1000; t1++ )
                    {
                        USBDeviceTasks();
                        CDCTxService();
                        if ( mUSBUSARTIsTxTrfReady() )
                        {
                            putUSBUSART( &data, 1 );
                            do {
                                USBDeviceTasks();
                                CDCTxService();
                            }while ( !USBUSARTIsTxTrfReady() );
                            break;
                        }
                    }
                }
            }
            break;
        default:
            return;
    }

    responce( pInfo, cmdbuf[0] );

}

static void PhaseGetData( tRemoteInfo* pInfo )
{
    BYTE data;
    BOOL bIsCmd = FALSE;
    BYTE datasize;
    BYTE cmdbuf[32];

    if ( GetByteFromNetwork( pInfo, &data ) )
    {
        cmdbuf[0] = data;

        switch ( data )
        {
            case SIO_REGISTER:
                if ( 8 != GetArrayFromNetwork( pInfo, cmdbuf + 1, sizeof(cmdbuf) - 1 ) )
                {
                    return;
                }
                DEBUGLOGRS_UART( "CMD REGISTER\r\n" );
                bIsCmd = TRUE;
                break;
            case SIO_TRANSMIT:
                recv_len = GetArrayFromNetwork( pInfo, cmdbuf + 1, sizeof(cmdbuf) - 1 );
                if ( recv_len == 0 )
                {
                    return;
                }
                bIsCmd = TRUE;
                break;
            case SIO_UNREGISTER:
                DEBUGLOGRS_UART( "CMD UNREGISTER\r\n" );
                bIsCmd = TRUE;
                break;
            case SIO_GET_STATUS:
                DEBUGLOGRS_UART( "CMD GET STATUS\r\n" );
                bIsCmd = TRUE;
                break;
            default:
                return;
        }
    }

//    StackTask();

    if ( MODE_SIO == pInfo->mode )
    {
        // SERIAL
        while ( rx_size < MAX_RXLEN )
        {
//            if ( DataRdyUART() )
            if ( DataRdySIO() )
            {
//                ser_rxbuffer[rx_size++] = ReadUART();
                ser_rxbuffer[rx_size++] = ReadSIO();
            }
            else
            {
                break;
            }
        }
    }

    if ( bIsCmd )
    {
        sioCommand( pInfo, cmdbuf );
    }
    else
    {
        responce( pInfo, SIO_RECEIVE );
    }

    if ( SIO_UNREGISTER == data )
    {
        pInfo->flags.stateNum = PHASE_LISTEN;
        DelayMs( 100 );
    }
}

