/*********************************************************************
 *
 *  BOOT Configuration Menu
 *    - BOOT configuration
 *    - XMODEM uploads of MPFS classic image
 *
 *********************************************************************
 * FileName:        BOOTConfig.c
 * Dependencies:    TCP/IP stack
 * Processor:       PIC18, PIC24F, PIC24H, dsPIC30F, dsPIC33F, PIC32
 * Compiler:        Microchip C32 v1.05 or higher
 *                  Microchip C30 v3.12 or higher
 *                  Microchip C18 v3.30 or higher
 *                  HI-TECH PICC-18 PRO 9.63PL2 or higher
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * Copyright (C) 2002-2009 Microchip Technology Inc.  All rights
 * reserved.
 *
 * Microchip licenses to you the right to use, modify, copy, and
 * distribute:
 * (i)  the Software when embedded on a Microchip microcontroller or
 *      digital signal controller product ("Device") which is
 *      integrated into Licensee's product; or
 * (ii) ONLY the Software driver source files ENC28J60.c, ENC28J60.h,
 *      ENCX24J600.c and ENCX24J600.h ported to a non-Microchip device
 *      used in conjunction with a Microchip ethernet controller for
 *      the sole purpose of interfacing with the ethernet controller.
 *
 * You should refer to the license agreement accompanying this
 * Software for additional information regarding your rights and
 * obligations.
 *
 * THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * MICROCHIP BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE
 * THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER
 * SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.
 *
 *
 * Author               Date      Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * E. Wood              4/26/08   Moved from MainDemo.c
 ********************************************************************/
#define __BOOTCONFIG_C

#include <ctype.h>
#include "TCPIPConfig.h"

#if defined(STACK_USE_UART)

#include "TCPIP Stack/TCPIP.h"
#include "usb_config.h"
#include "./USB/usb.h"
#include "./USB/usb_function_cdc.h"
#include "MainDemo.h"

extern BOOL CalcSum( BYTE* data, WORD len, BOOL bWrite );

void putsBOOT( const char* str );
void WriteBOOT( BYTE chr );
void WaitCDCTxRdy( void );
BOOL BusyBOOT( void );
void WriteBOOT( BYTE chr );

static BOOL DataRdyBOOT( void );
static void putrsBOOT( const rom char* str );
static BYTE ReadStringBOOT( BYTE *Dest, BYTE BufferLen );
static BYTE ReadBOOT( void );
static BYTE ReadStringBOOT( BYTE *Dest, BYTE BufferLen );

enum
{
    BOOT_ROOT_SIO,
    BOOT_ROOT_CDC,
    BOOT_ROOT_BOTH
};

enum
{
    READ_NONE,
    READ_IP,
    READ_PORT,
    READ_STR
};

static BYTE g_root = BOOT_ROOT_BOTH;
static BOOL g_cdcen = TRUE;

BYTE gLastCDCByte;
BOOL gLastCDCRx = FALSE;

#if (defined(MPFS_USE_EEPROM) || defined(MPFS_USE_SPI_FLASH)) && defined(STACK_USE_MPFS)
    static BOOL DownloadMPFS(void);
#endif

#define MAX_USER_RESPONSE_LEN   (20u)


void printMac( void )
{
    BYTE i;
    BYTE str[20];
    BYTE* pt = str;

    for(i = 0; i < 6u; i++)
    {
        if( i )
        {
            *pt++ = ':';
        }
        *pt++ = btohexa_high( AppConfig.MyMACAddr.v[i] );
        *pt++ = btohexa_low( AppConfig.MyMACAddr.v[i] );
    }
    *pt = '\0';
    putsBOOT( str);
//    putrsBOOT("\r\n");
}

void printPort( WORD port )
{
    BYTE str[6];

    sprintf( str, "%u", port );
    putsBOOT( str);
}

BOOL BusyBOOT( void )
{
    BOOL bResult = FALSE;

    switch ( g_root )
    {
    case BOOT_ROOT_SIO:
        bResult = BusyUART();
        break;
    case BOOT_ROOT_CDC:
        if ( g_cdcen )
        {
            bResult = !mUSBUSARTIsTxTrfReady();
            if ( bResult )
            {
                CDCTxService();
            }
        }
        break;
    case BOOT_ROOT_BOTH:
    default:
        bResult = BusyUART();
        if ( bResult && g_cdcen )
        {
            bResult = !mUSBUSARTIsTxTrfReady();
            if ( bResult )
            {
                CDCTxService();
            }
        }
        break;
    }
    return bResult;
}

void WaitCDCTxRdy( void )
{
    while ( !mUSBUSARTIsTxTrfReady() )
    {
        USBDeviceTasks();
        CDCTxService();
    }
}

static BYTE ReadStringBOOT( BYTE *Dest, BYTE BufferLen )
{
    BYTE c;
    BYTE count = 0;

    while( BufferLen-- )
    {
        *Dest = '\0';

        while ( !DataRdyBOOT() );
        c = ReadBOOT();

        if( c == '\r' || c == '\n' )
        {
            break;
        }
        else if ( c == 0x08 || c == 0x7f )
        {
            if ( count )
            {
                count--;
                Dest--;
                while (BusyBOOT())
                    ;
                WriteBOOT( 0x08 );

                while (BusyBOOT())
                    ;
                WriteBOOT( ' ' );

                while (BusyBOOT())
                    ;
                WriteBOOT( 0x08 );
            }
            continue;
        }
        else if ( !isprint( c ) )
        {
            continue;
        }

        while (BusyBOOT())
            ;
        WriteBOOT( c );  //echo

        count++;
        *Dest++ = c;
    }

    return count;
}

static void putrsBOOT( const rom char* str )
{
    switch ( g_root )
    {
    case BOOT_ROOT_SIO:
        putrsUART( str );
        break;
    case BOOT_ROOT_CDC:
        if ( g_cdcen )
        {
            WaitCDCTxRdy();
            putrsUSBUSART( str );
            WaitCDCTxRdy();
        }
        break;
    case BOOT_ROOT_BOTH:
    default:
        putrsUART( str );
        if ( g_cdcen )
        {
            WaitCDCTxRdy();
            putrsUSBUSART( str );
            WaitCDCTxRdy();
        }
        break;
    }
}

void putsBOOT( const char* str )
{
    switch ( g_root )
    {
    case BOOT_ROOT_SIO:
        putsUART( (unsigned char*)str );
        break;
    case BOOT_ROOT_CDC:
        if ( g_cdcen )
        {
            WaitCDCTxRdy();
            putsUSBUSART( (unsigned char*)str );
            WaitCDCTxRdy();
        }
        break;
    case BOOT_ROOT_BOTH:
    default:
        putsUART( (unsigned char*)str );
        if ( g_cdcen )
        {
            WaitCDCTxRdy();
            putsUSBUSART( (unsigned char*)str );
            WaitCDCTxRdy();
        }
        break;
    }
}

static BOOL DataRdyBOOT( void )
{
    BOOL bResult = FALSE;

    switch ( g_root )
    {
    case BOOT_ROOT_SIO:
//        bResult = DataRdyUART();
        bResult = DataRdySIO();
        break;
    case BOOT_ROOT_CDC:
        if ( g_cdcen )
        {
            if ( gLastCDCRx )
            {
                bResult = TRUE;
            }
            else
            {
                if ( getsUSBUSART( &gLastCDCByte, 1 ) )
                {
                    gLastCDCRx = TRUE;
                    bResult = TRUE;
                }
            }
        }
        break;
    case BOOT_ROOT_BOTH:
    default:
//        bResult = DataRdyUART();
        bResult = DataRdySIO();
        if ( !bResult && g_cdcen )
        {
            if ( gLastCDCRx )
            {
                bResult = TRUE;
            }
            else
            {
                if ( getsUSBUSART( &gLastCDCByte, 1 ) )
                {
                    gLastCDCRx = TRUE;
                    bResult = TRUE;
                }
            }
        }
        break;
    }

    return bResult;
}


static BYTE ReadBOOT( void )
{
    BYTE result;

    switch ( g_root )
    {
    case BOOT_ROOT_SIO:
//        result = ReadUART();
        result = ReadSIO();
        break;
    case BOOT_ROOT_CDC:
        if ( g_cdcen )
        {
            result= 0x00;
            if ( gLastCDCRx )
            {
                result = gLastCDCByte;
                gLastCDCRx = FALSE;
            }
            else
            {
                getsUSBUSART( &result, 1 );
            }
        }
        break;
    case BOOT_ROOT_BOTH:
    default:
//        result = ReadUART();
        result = ReadSIO();
        if ( !isprint(result) )
        {
            if ( g_cdcen )
            {
                result= 0x00;
                if ( gLastCDCRx )
                {
                    result = gLastCDCByte;
                    gLastCDCRx = FALSE;
                }
                else
                {
                    getsUSBUSART( &result, 1 );
                }

                if ( isprint(result) )
                {
                    g_root = BOOT_ROOT_CDC;
                    sprintf( &LCDText[16], "CDC Data Rx Mode" );
                    LCDUpdate();
                }
            }
        }
        else
        {
            g_root = BOOT_ROOT_SIO;
            sprintf( &LCDText[16], "SIO Data Rx Mode" );
            LCDUpdate();
        }
        break;
    }

    return result;
}

void WriteBOOT( BYTE chr )
{
    switch ( g_root )
    {
    case BOOT_ROOT_SIO:
        WriteUART( chr );
        break;
    case BOOT_ROOT_CDC:
        if ( g_cdcen )
        {
            putUSBUSART( &chr, 1 );
            WaitCDCTxRdy();
        }
        break;
    case BOOT_ROOT_BOTH:
    default:
        WriteUART( chr );
        if ( g_cdcen )
        {
            putUSBUSART( &chr, 1 );
            WaitCDCTxRdy();
        }
        break;
    }
}

// Writes an IP address to the UART display and the UART as available
void DisplayIPValueUART( IP_ADDR IPVal )
{
    BYTE IPDigit[4];
    BYTE i;
    BYTE j;

    for(i = 0; i < sizeof(IP_ADDR); i++)
    {
        uitoa((WORD)IPVal.v[i], IPDigit);

        putsBOOT( IPDigit );

        if(i == sizeof(IP_ADDR)-1)
            break;

        putrsBOOT( "." );
    }
}

void DoUARTConfig(void)
{
    #define DELAY_TIME  30000

    BYTE response[MAX_USER_RESPONSE_LEN];
    IP_ADDR tempIPValue;
    IP_ADDR *destIPValue;
    WORD*   destPort;
    long    ltemp;
    BYTE    status;
    char*   destStr;
    WORD    delay;
    char    version[10];

    CDCSetBaudRate( 19200 );
    CDCSetDataSize( 8 );
    CDCSetParity( PARITY_NONE );
    CDCSetCharacterFormat( NUM_STOP_BITS_1 );

    for ( ;; )
    {
        USBDeviceTasks(); // Interrupt or polling method.  If using polling, must call

        // USB CDC Service
        if( (USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1) )
        {
            LED1_IO = 0;
            g_cdcen = FALSE;
            break;
        }
        else
        {
            LED1_IO = 1;
            g_cdcen = TRUE;
            break;
        }
    }

    sprintf( version, "%d.%d", PICNIC_VER1, PICNIC_VER2 );

//    while(!bQuit)
    for ( ;; )
    {
        USBDeviceTasks(); // Interrupt or polling method.  If using polling, must call

        // Display the menu
        putrsBOOT( "\r\n\r\nPIC-NIC Configuration (MAC Address = " );
        printMac();
        putrsBOOT( ")\r\n" );
        putrsBOOT( "PIC-NIC ver " );
        putsBOOT( version );
        putrsBOOT( " / Microchip TCP/IP Stack : " VERSION "\r\n" );

        putrsBOOT( "\r\n\t[I]: IP address:\t\t" );
        DisplayIPValueUART(AppConfig.DefaultIPAddr);

        putrsBOOT( "\r\n\t[G]: Gateway address:\t\t" );
        DisplayIPValueUART(AppConfig.DefaultGateway);

        putrsBOOT( "\r\n\t[P]: DNS address 1:\t\t" );
        DisplayIPValueUART(AppConfig.PrimaryDNSServer);

        putrsBOOT( "\r\n\t[S]: DNS address 2:\t\t" );
        DisplayIPValueUART(AppConfig.SecondaryDNSServer);

        putrsBOOT( "\r\n\t[N]: Net mask:\t\t\t" );
        DisplayIPValueUART(AppConfig.DefaultMask);

        putrsBOOT( "\r\n\t[T]: NTP server:\t\t" );
        putsBOOT( gNTPServerName.name );

        putrsBOOT( "\r\n\t[H]: HTTP port:\t\t\t" );
        printPort( AppConfig.HTTPPort );

        putrsBOOT( "\r\n\t[L]: LCD port:\t\t\t" );
        printPort( AppConfig.LCDPort );

        putrsBOOT( "\r\n\t[Q]: Parallel port:\t\t" );
        printPort( AppConfig.PARALLELPort );

        putrsBOOT( "\r\n\t[R]: Serial port:\t\t" );
        printPort( AppConfig.SERIALPort );

        putrsBOOT( "\r\n\t[C]: CDC port:\t\t\t" );
        printPort( AppConfig.CDCPort );

        putrsBOOT( "\r\n\t[K]: Password:\t\t\t" );
        putsBOOT( AuthConfig.password );

        putrsBOOT("\r\n\t[0]: ");
        putrsBOOT((ROM BYTE*)(AppConfig.Flags.bIsDHCPEnabled ? "Dis" : "En"));
        putrsBOOT("able DHCP :\t\tDHCP is currently ");
        putrsBOOT((ROM BYTE*)(AppConfig.Flags.bIsDHCPEnabled ? "Enabled" : "Disabled"));

        putrsBOOT("\r\n\t[1]: ");
        putrsBOOT("LCD port to ");
        putrsBOOT((ROM BYTE*)(AppConfig.Flags.bLcdPortIsTcp ? "UDP" : "TCP"));
        putrsBOOT(" :\t\tLCD port is currently ");
        putrsBOOT((ROM BYTE*)(AppConfig.Flags.bLcdPortIsTcp ? "TCP" : "UDP"));

        putrsBOOT("\r\n\t[2]: ");
        putrsBOOT("Parallel port to ");
        putrsBOOT((ROM BYTE*)(AppConfig.Flags.bParallelPortIsTcp ? "UDP" : "TCP"));
        putrsBOOT(" :\t\Parallel port is currently ");
        putrsBOOT((ROM BYTE*)(AppConfig.Flags.bParallelPortIsTcp ? "TCP" : "UDP"));

        putrsBOOT("\r\n\t[3]: ");
        putrsBOOT("Serial port to ");
        putrsBOOT((ROM BYTE*)(AppConfig.Flags.bSerialPortIsTcp ? "UDP" : "TCP"));
        putrsBOOT(" :\t\Serial port is currently ");
        putrsBOOT((ROM BYTE*)(AppConfig.Flags.bSerialPortIsTcp ? "TCP" : "UDP"));

        putrsBOOT("\r\n\t[4]: ");
        putrsBOOT("CDC port to ");
        putrsBOOT((ROM BYTE*)(AppConfig.Flags.bCDCPortIsTcp ? "UDP" : "TCP"));
        putrsBOOT(" :\t\t\CDC port is currently ");
        putrsBOOT((ROM BYTE*)(AppConfig.Flags.bCDCPortIsTcp ? "TCP" : "UDP"));

        putrsBOOT("\r\n\t[5]: ");
        putrsBOOT((ROM BYTE*)(AppConfig.Flags.bIsAuth ? "Dis" : "En"));
        putrsBOOT("able Authentication :\tAuthentication is currently ");
        putrsBOOT((ROM BYTE*)(AppConfig.Flags.bIsAuth ? "Enabled" : "Disabled"));

        putrsBOOT( "\r\n\t[A]: All initialize" );

        putrsBOOT("\r\n\t[!]: Save Configuration");
        putrsBOOT("\r\nEnter a menu choice: ");

        // Wait for the user to press a key
        while( !DataRdyBOOT() )
        {
            USBDeviceTasks(); // Interrupt or polling method.  If using polling, must call
            // USB CDC Service
            if( (USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1) )
            {
                LED1_IO = 0;
                g_cdcen = FALSE;
            }
            else
            {
                LED1_IO = 1;
                CDCTxService();
                g_cdcen = TRUE;
            }
        }

        putrsBOOT((ROM char*)"\r\n");

        status = READ_NONE;

        // Execute the user selection
        switch( ReadBOOT() )
        {
            case 'I':
                putrsBOOT( "IP Address : " );
                destIPValue = &AppConfig.DefaultIPAddr;
                status = READ_IP;
                break;

            case 'G':
                putrsBOOT( "Gateway : ");
                destIPValue = &AppConfig.DefaultGateway;
                status = READ_IP;
                break;

            case 'P':
                putrsBOOT( "DNS 1 : ");
                destIPValue = &AppConfig.PrimaryDNSServer;
                status = READ_IP;
                break;

            case 'S':
                putrsBOOT( "DNS 2 : ");
                destIPValue = &AppConfig.SecondaryDNSServer;
                status = READ_IP;
                break;

            case 'N':
                putrsBOOT( "Netmask : ");
                destIPValue = &AppConfig.DefaultMask;
                status = READ_IP;
                break;

            case 'T':
                putrsBOOT( "NTP Server : ");
                destStr = gNTPServerName.name;
                status = READ_STR;
                break;

            case 'K':
                putrsBOOT( "Auth Password : ");
                destStr = (char*)AuthConfig.password;
                status = READ_STR;
                break;

            case 'H':
                putrsBOOT( "HTTP Port : ");
                destPort = &AppConfig.HTTPPort;
                status = READ_PORT;
                break;

            case 'L':
                putrsBOOT( "LCD Port : ");
                destPort = &AppConfig.LCDPort;
                status = READ_PORT;
                break;

            case 'Q':
                putrsBOOT( "Parallel Port : ");
                destPort = &AppConfig.PARALLELPort;
                status = READ_PORT;
                break;

            case 'R':
                putrsBOOT( "Serial Port : ");
                destPort = &AppConfig.SERIALPort;
                status = READ_PORT;
                break;

            case 'C':
                putrsBOOT( "CDC Port : ");
                destPort = &AppConfig.CDCPort;
                status = READ_PORT;
                break;

            case '0':
                AppConfig.Flags.bIsDHCPEnabled = !AppConfig.Flags.bIsDHCPEnabled;
                break;

            case '1':
                AppConfig.Flags.bLcdPortIsTcp = !AppConfig.Flags.bLcdPortIsTcp;
                break;

            case '2':
                AppConfig.Flags.bParallelPortIsTcp = !AppConfig.Flags.bParallelPortIsTcp;
                break;

            case '3':
                AppConfig.Flags.bSerialPortIsTcp = !AppConfig.Flags.bSerialPortIsTcp;
                break;

            case '4':
                AppConfig.Flags.bCDCPortIsTcp = !AppConfig.Flags.bCDCPortIsTcp;
                break;

            case '5':
                AppConfig.Flags.bIsAuth = !AppConfig.Flags.bIsAuth;
                AuthConfig.authflag = AppConfig.Flags.bIsAuth;
                break;

            case 'A':
                InitAppConfigSub( &AppConfig );
                strcpypgm2ram( gNTPServerName.name, (ROM char*)DEFAULT_NTP_SERVER );
                strcpypgm2ram( (char*)AuthConfig.password, (ROM char*)MY_DEFAULT_AUTH_PASSWORD );
                AppConfig.Flags.bIsAuth = 0;
                AuthConfig.authflag = 0;
                break;

            case '!':
                #if defined(EEPROM_CS_TRIS)
                    SaveAppConfig();
                    putrsBOOT( "Settings saved.\r\n" );

                    // wait
                    for ( delay = DELAY_TIME; delay > 0; /**/ )
                    {
                        delay--;
                    }

                    CalcSum( (BYTE*)&AuthConfig, sizeof(AuthConfig) - sizeof(AuthConfig.sum), TRUE );
                    XEEBeginWrite( AUTH_OFFSET );
                    XEEWriteArray( (BYTE*)&AuthConfig, sizeof(AuthConfig) );
                    putrsBOOT( "Auth saved.\r\n" );

                    // wait
                    for ( delay = DELAY_TIME; delay > 0; /**/ )
                    {
                        delay--;
                    }

                    CalcSum( (BYTE*)&gNTPServerName, sizeof(gNTPServerName) - sizeof(gNTPServerName.sum), TRUE );
                    XEEBeginWrite( NTP_OFFSET );
                    XEEWriteArray( (BYTE*)&gNTPServerName, sizeof(gNTPServerName) );
                    putrsBOOT( "NTP saved.\r\n" );
                #else
                    putrsBOOT("External EEPROM/Flash not present -- settings will be lost at reset.\r\n");
                #endif
                break;

        }

        if ( READ_IP == status )
        {
            ReadStringBOOT( response, sizeof(response) );

            if( StringToIPAddress(response, &tempIPValue) )
            {
                destIPValue->Val = tempIPValue.Val;
            }
            else
            {
                putrsBOOT("Invalid input.\r\n");
            }
        }
        else if ( READ_PORT == status )
        {
            ReadStringBOOT( response, sizeof(response) );
            ltemp = atol( response );

            if( ltemp < 0 || 65535 < ltemp )
            {
                putrsBOOT("Invalid input.\r\n");
            }
            else
            {
                *destPort = (WORD)ltemp;
            }
        }
        else if ( READ_STR == status )
        {
            ReadStringBOOT( response, sizeof(response) );
            strcpy( destStr, response );
        }
    }
}


#if defined(MPFS_USE_EEPROM) && defined(STACK_USE_MPFS)
/*********************************************************************
 * Function:        BOOL DownloadMPFS(void)
 *
 * PreCondition:    MPFSInit() is already called.
 *
 * Input:           None
 *
 * Output:          TRUE if successful
 *                  FALSE otherwise
 *
 * Side Effects:    This function uses 128 bytes of Bank 4 using
 *                  indirect pointer.  This requires that no part of
 *                  code is using this block during or before calling
 *                  this function.  Once this function is done,
 *                  that block of memory is available for general use.
 *
 * Overview:        This function implements XMODEM protocol to
 *                  be able to receive a binary file from PC
 *                  applications such as HyperTerminal.
 *
 * Note:            In current version, this function does not
 *                  implement user interface to set IP address and
 *                  other informations.  User should create their
 *                  own interface to allow user to modify IP
 *                  information.
 *                  Also, this version implements simple user
 *                  action to start file transfer.  User may
 *                  evaulate its own requirement and implement
 *                  appropriate start action.
 *
 ********************************************************************/
#define XMODEM_SOH      0x01u
#define XMODEM_EOT      0x04u
#define XMODEM_ACK      0x06u
#define XMODEM_NAK      0x15u
#define XMODEM_CAN      0x18u
#define XMODEM_BLOCK_LEN 128u
//////////////////////////////////////////////////////////////////////////////////////////
// NOTE: The following XMODEM code pretains to MPFS Classic.
//       Upgrading to HTTP2 and MPFS2 is *strongly* recommended for all new designs.
//       MPFS2 images can be uploaded directly using the MPFS2.exe tool.
//////////////////////////////////////////////////////////////////////////////////////////
static BOOL DownloadMPFS(void)
{
    enum SM_MPFS
    {
        SM_MPFS_SOH,
        SM_MPFS_BLOCK,
        SM_MPFS_BLOCK_CMP,
        SM_MPFS_DATA,
    } state;

    BYTE c;
    MPFS handle;
    BOOL lbDone;
    BYTE blockLen;
    BYTE lResult;
    BYTE tempData[XMODEM_BLOCK_LEN];
    TICK lastTick;
    TICK currentTick;

    state = SM_MPFS_SOH;
    lbDone = FALSE;

    handle = MPFSFormat();

    // Notify the host that we are ready to receive...
    lastTick = TickGet();
    do
    {
        currentTick = TickGet();
        if ( TickGetDiff(currentTick, lastTick) >= (TICK_SECOND/2) )
        {
            lastTick = TickGet();
            while(BusyBOOT());
            WriteBOOT(XMODEM_NAK);

            /*
             * Blink LED to indicate that we are waiting for
             * host to send the file.
             */
            LED6_IO ^= 1;
        }

    } while( !DataRdyBOOT() );


    while(!lbDone)
    {
        if( DataRdyBOOT() )
        {
            // Toggle LED as we receive the data from host.
            LED6_IO ^= 1;
            c = ReadBOOT();
        }
        else
        {
            // Real application should put some timeout to make sure
            // that we do not wait forever.
            continue;
        }

        switch(state)
        {
        default:
            if ( c == XMODEM_SOH )
            {
                state = SM_MPFS_BLOCK;
            }
            else if ( c == XMODEM_EOT )
            {
                // Turn off LED when we are done.
                LED6_IO = 1;

                MPFSClose();
                while(BusyBOOT());
                WriteBOOT(XMODEM_ACK);
                lbDone = TRUE;
            }
            else
            {
                while(BusyBOOT());
                WriteBOOT(XMODEM_NAK);
            }

            break;

        case SM_MPFS_BLOCK:

            // We do not use block information.
            lResult = XMODEM_ACK;
            blockLen = 0;
            state = SM_MPFS_BLOCK_CMP;
            break;

        case SM_MPFS_BLOCK_CMP:

            // We do not use 1's comp. block value.
            state = SM_MPFS_DATA;
            break;

        case SM_MPFS_DATA:

            // Buffer block data until it is over.
            tempData[blockLen++] = c;
            if ( blockLen > XMODEM_BLOCK_LEN )
            {

                // We have one block data. Write it to EEPROM.
                MPFSPutBegin(handle);

                lResult = XMODEM_ACK;
                for ( c = 0; c < XMODEM_BLOCK_LEN; c++ )
                    MPFSPut(tempData[c]);

                handle = MPFSPutEnd();

                while(BusyBOOT());
                WriteBOOT(lResult);
                state = SM_MPFS_SOH;
            }
            break;

        }

    }

    return TRUE;
}
#endif  // #if defined(MPFS_USE_EEPROM) && defined(STACK_USE_MPFS)

#endif //#if defined(STACK_USE_UART)
